Package: SPAS
Type: Package
Title: Stratified-Petersen Analysis System
Version: 2026.4.1
Date: 2026-04-01
Authors@R: person(given = c("Carl", "James"),
                    family = "Schwarz",
                    role = c("aut", "cre"),
                    email = "cschwarz.stat.sfu.ca@gmail.com")
LinkingTo: TMB, RcppEigen
Imports: checkmate, MASS, Matrix, msm, numDeriv, plyr, reshape2, TMB
        (>= 1.7.15), utils
Description: The Stratified-Petersen Analysis System (SPAS) is designed
    to estimate abundance in two-sample capture-recapture experiments 
    where the capture and recaptures are stratified. This is a generalization
    of the simple Lincoln-Petersen estimator.
    Strata may be defined in time or in space or both, 
    and the s strata in which marking takes place 
    may differ from the t strata in which recoveries take place.
    When s=t, SPAS reduces to the method described by 
    Darroch (1961) <doi:10.2307/2332748>.
    When s<t, SPAS implements the methods described in
    Plante, Rivest, and Tremblay (1988) <doi:10.2307/2533994>.
    Schwarz and Taylor (1998) <doi:10.1139/f97-238> describe
    the use of SPAS in estimating return of salmon stratified by
    time and geography. 
    A related package, BTSPAS, deals with temporal stratification where 
    a spline is used to model the distribution of the population 
    over time as it passes the second capture location.
    This is the R-version of the (now obsolete) standalone Windows 
    program of the same name.
License: GPL (>= 2)
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/cschwarz-stat-sfu-ca/SPAS
BugReports: https://github.com/cschwarz-stat-sfu-ca/SPAS/issues
NeedsCompilation: yes
Packaged: 2026-03-26 20:37:34 UTC; cschwarz
Author: Carl James Schwarz [aut, cre]
Maintainer: Carl James Schwarz <cschwarz.stat.sfu.ca@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-27 17:40:17 UTC

# Generated by roxygen2: do not edit by hand

export(SPAS.autopool)
export(SPAS.fit.model)
export(SPAS.print.model)
import(MASS)
import(TMB)
import(msm)
import(plyr)
import(stats)
importFrom(Matrix,bdiag)
importFrom(checkmate,assert_logical)
importFrom(reshape2,melt)
importFrom(utils,packageVersion)
useDynLib(SPAS)

# SPAS
Stratified Petersen Analysis System in R

## Versions and installation

  * **CRAN**  Download the **SPAS** package

  * **Github** To install the latest development version from Github, 
    install the newest version of the **devtools** package; then run
```
devtools::install_github("cschwarz-stat-sfu-ca/SPAS", dependencies = TRUE,
                        build_vignettes = TRUE)
```

## Features
This is an R version of the Windoze program SPAS to estimate population abundance
using a Stratified Petersen Estimator (Darroch 1961; Plante et al 1998; Schwarz and Taylor, 1998)

The user is allows to pool rows and/or columns prior to analysis but the number of rows must be
less than or equal to the number of columns (s <= t). The conditional likelihood formulation of
Plante et al (1998) is used to estimate the parameters. 

A good discussion 
of how to decide on pooling rows/columns is found in Schwarz and Taylor (1998). The *row.physical.pool*
parameter allows you to choose between physical pooling of rows, or logical pooling of rows (the underlying
data table is unchanged, but capture probabilities for the pooled rows are forced equal). It is not possible
to do logical pooling of columns and only physical pooling is possible. See the help() function 
for details.

If the data are physically pooled prior to analysis, it is not possible to compare different
poolings to see which is most appropriate using AIC or likelihood ratio tests. If you do logical 
pooling of rows, you can compare poolings using AIC or likelihood ratio methods.

Optimization is now done using the TMB package (a relative of ADMB) which seems to have
fixed the convergence issues that plagued earlier versions of SPAS-R.

## References
Darroch, J. N. (1961). The two-sample capture-recapture census when tagging and sampling are stratified. Biometrika, 48, 241-260.
https://www.jstor.org/stable/2332748

Plante, N., L.-P Rivest, and G. Tremblay. (1988). Stratified Capture-Recapture Estimation of the Size of a Closed Population. Biometrics 54, 47-60.
https://www.jstor.org/stable/2533994

Schwarz, C. J., & Taylor, C. G. (1998). The use of the stratified-Petersen estimator in fisheries management with an illustration of estimating the number of pink salmon (Oncorhynchus gorbuscha) that return to spawn in the Fraser River. Canadian Journal of Fisheries and Aquatic Sciences, 55, 281-296.
https://doi.org/10.1139/f97-238

# Utility functions used in this package.
# These functions will not be visible to the user.


#' @param variable Variable to be checked if numeric
#' @param min.value Minimum value for variable
#' @param max.value Maximum value for variable
#' @param req.length Required length for variable
#' @param check.whole Check if all values in variable are whole numbers
#' @param data A  data frame
#' @param dt_type Type of double tags. See valid_dt_type()
#' @param sep. Separator for capture histories

#'
#' @importFrom reshape2 melt
#' @noRd



#######################################################################3
# Check that argument is numeric, specified length, and all values are between min /max
is.wholenumber <- function(x, tol = .Machine$double.eps^0.5){
  abs(x - round(x)) < tol}  # taken from examples of is.integer


#######################################################################3

check.numeric <- function(variable, min.value=0, max.value=Inf, req.length=1, check.whole=TRUE){
  var.name <- deparse(substitute(variable))
  if(!is.numeric(min.value) | !is.numeric(max.value) | !is.numeric(req.length))
    stop("Bad input values to check.numeric: ", paste(c(var.name,"; ", min.value,"; ", max.value,"; ", req.length), collapse=", "))
  if(length(req.length) !=1)stop("Bad required length for check.numeric. You had ", req.length)
  if(length(min.value) !=1 & length(min.value) != req.length)
    stop("Min values wrong length in check.numeric. You have: ",paste(min.value, collapse=", "))
  if(length(max.value) !=1 & length(max.value) != req.length)
    stop("Max values wrong length in check.numeric. You have: ",paste(max.value, collapse=", "))
  if(!is.numeric(variable))stop("'",var.name, "' must be numeric")
  if( any(is.na(variable)))stop("'",var.name, "' cannot be missing. You have ", paste(variable, collapse=", "))
  if( length(variable) != req.length)stop("'",var.name, "' must have length ", req.length)
  if( any(variable < min.value) | any(variable > max.value))
    stop("'",var.name, "' must between (",paste(min.value, collapse=", "),") and (",
         paste(max.value, collapse=", "),
         ') (inclusive). You have ',paste(variable, collapse=", "))
  # check if values are all whole numbers
  if(check.whole){
    if(any(!is.wholenumber(variable)))stop("'",var.name, "' must be integers. You had ", paste(variable, collapse=", "))
  }
  invisible()
}

#' Print or Extract the results from a fit of a Stratified-Petersen (SP) model when using the TMB optimizer
#' 
#' This function makes a report/extracts the components of the results of the model fitting
#' .
#' @param x A result from the model fitting. See \code{SPAS.fit.model}
#' @param extract Should the function simply return the "printed" components in a list?.

#' @return A report to the console or extracted components. Refer to the vignettes.
#' 
#' @importFrom checkmate assert_logical 

#' @export
#' @examples 
#' conne.data.csv <- textConnection("
#' 9  ,    21  ,     0  ,    0  ,    0  ,    0  ,   171
#' 0  ,   101  ,    22  ,    1  ,    0  ,    0  ,   763
#' 0  ,     0  ,   128  ,   49  ,    0  ,    0  ,   934
#' 0  ,     0  ,     0  ,   48  ,   12  ,    0  ,   434
#' 0  ,     0  ,     0  ,    0  ,    7  ,    0  ,    49
#' 0  ,     0  ,     0  ,    0  ,    0  ,    0  ,     4
#' 351,  2736  ,  3847  , 1818  ,  543  ,   191 ,     0")
#' conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
#' close(conne.data.csv)
#'  
#' mod1 <- SPAS.fit.model(conne.data, model.id="Pooling rows 1/2, 5/6; pooling columns 5/6",
#'                       row.pool.in=c("12","12","3","4","56","56"),
#'                       col.pool.in=c(1,2,3,4,56,56))
#'
#' SPAS.print.model(mod1)
#' 
#' out <- SPAS.print.model(mod1, extract=TRUE)
#' names(out)
#' out$spas




SPAS.print.model = function(x, extract=FALSE){
#
#  Print out/extract model results from the OPEN SPAS model fit using TMB
#
#  Check that object is of the new form
   if( x$version<"SPAS-R 2020-01-01"){
       stop("Object was created using previous version of SPAS. Use SPAS.print.model.legacy()")
   }
  
   # check arguments
   checkmate::assert_logical(extract, any.missing=FALSE, len=1)
  
   res <- list()
   
   if(!extract){
     cat("Model Name:",x$model.info$model.id,"\n")
   cat("   Date of Fit:",format(x$date,"%Y-%m-%d %H:%M"),"\n")
   cat("   Version of OPEN SPAS used :", x$version,"\n")
   cat(" \n")
   cat("Raw data \n")
   print(x$input$rawdata)
   cat("\n")
   cat("Row pooling setup :", x$input$row.pool.in, "\n")
   cat("Col pooling setup :", x$input$col.pool.in, "\n")
   cat("Physical pooling  :", x$input$row.physical.pool, "\n")
   cat("Theta pooling     :", x$input$theta.pool,  "\n")
   cat("CJS pooling       :", x$input$CJSpool,     "\n")
   cat("\n\n")
   }
   if(extract){
     res$model_name = x$model.info$model.id
     res$date <- x$date
     res$version <- x$version
     res$input <- x$input 
   }
   
   if(!extract){
     cat("Chapman estimator of population size ",round(x$est$N.Chapman),
         ' (SE ', round(x$se$N.Chapman), " )\n")
     cat(" \n\n")
   }
   if(extract){
     res$Chapman$est <- x$est$N.Chapman
     res$Chapman$se  <- x$se $N.Chapman
   }
   
   if(!extract){
     cat("Raw data AFTER PHYSICAL (but not logical) POOLING \n")
     print(x$fit.setup$pooldata)
   }
   if( extract){
     res$fit <- NULL
     res$fit$pooldata <- x$fit.setup$pooldata
   }
   
   if(!extract){
      cat("\n")
      cat("Condition number of XX' where X= (physically) pooled matrix is ", x$kappa, "\n")
      cat("Condition number of XX' after logical pooling                  ", x$kappa.after.lp, "\n")
      cat("\n")
      cat("Large value of kappa (>1000) indicate that rows are approximately proportional which is not good\n")
      cat("\n")
   }
   if( extract){
      res$fit$kappa <- x$kappa
      res$fit$kappa.after.lp <- x$kappa.after.lp
   }
   
   if(!extract){
      cat("  Conditional   Log-Likelihood:",x$model.info$logL.cond,"   ;  np:",x$model.info$np,";  AICc:", x$model.info$AICc,"\n")
      cat("\n")

      cat("  Code/Message from optimization is: ",
          x$conditional$res$optim.info$convergence,x$conditional$res$optim.info$message,"\n" )
   }
   if(extract){
     res$fit$model.info <- x$model.info
     res$fit$model.info$optim.info <- x$conditional$res$optim.info
   }
   
   
   # Get the estimates in a nice format
   temp <- x$fit.setup$pooldata   # get the basic matrix
   S <- nrow(temp)-1
   T <- ncol(temp)-1

   temp[1:S, 1:T]  <- round(x$est$real$theta,1)
   temp[1:S,   T+1]<- round(x$est$real$psi,1);                colnames(temp)[T+1] <- "psi"
   temp <- cbind(temp,round(c(x$est$real$cap,       NA),3));  colnames(temp)[T+2] <- "cap.prob"
   temp <- cbind(temp,round(c(x$est$real$exp.factor,NA),1));  colnames(temp)[T+3] <- "exp factor"
   temp[S+1,1:T] <-   round(as.vector((1-x$est$real$cap)/x$est$real$cap) %*% matrix(x$est$real$theta,nrow=S, ncol=T)) ;   rownames(temp)[S+1] <- "est unmarked" 
   temp<- cbind(temp, round(c(x$est$real$N.stratum, x$est$real$N)))     ; colnames(temp)[T+4] <- "Pop Est"
   
   if(!extract){
     cat("\nEstimates\n")
     print(temp)
   }
   if(extract){
     res$spas$estimate <- temp
   }
   
   # get the se's of the estimates
   temp <- x$fit.setup$pooldata
   temp[1:S, 1:T]  <- round(x$se$real$theta,1)
   temp[1:S,   T+1]<- round(x$se$real$psi,1);                 colnames(temp)[T+1] <- "psi"
   temp <- cbind(temp,round(c(x$se$real$cap,NA),3));          colnames(temp)[T+2] <- "cap.prob"
   temp <- cbind(temp,round(c(x$se$real$exp.factor,NA),1));   colnames(temp)[T+3] <- "exp factor"
   temp[S+1,1:T] <-   as.vector(rep(NA,T) )                    ;   rownames(temp)[S+1] <- "est unmarked" 
   temp<- cbind(temp, round(c(x$se$real$N.stratum, x$se$real$N))); colnames(temp)[T+4] <- "Pop Est"
   
   if(!extract){
     cat("\nSE of above estimates\n")
     print(temp)
   }
   if(extract){
     res$spas$se <- temp
   }
 
   if(!extract){  
      # goodness of fit statistics
      cat("\n\n")
      cat("Chisquare gof cutoff  :", x$input$chisq.cutoff, "\n")
      cat("Chisquare gof value   :", x$gof$chisq,          "\n")
      cat("Chisquare gof df      :", x$gof$chisq.df,       "\n")
      cat("Chisquare gof p       :", x$gof$chisq.p,        "\n")
   }
   if(extract){
      res$gof <- x$gof
      res$gof$chisq.cutoff <- x$input$chisq.cutoff
   }
   
   if(extract){
      return(res)
   }
   
 }

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------

test.data.csv <- textConnection("
  86   ,     54   ,     39   ,   219
  76   ,     35   ,     45   ,   168
  24   ,     53   ,     73   ,   190
1039   ,   1148   ,   2009   ,   0")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data

## -----------------------------------------------------------------------------
library(SPAS)
mod1 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:3, col.pool.in=1:3)

SPAS.print.model(mod1)

## -----------------------------------------------------------------------------
mod2 <- SPAS.fit.model(test.data,
                           model.id="Logical pooling to single row",
                           row.pool.in=c(1,1,1), col.pool.in=1:3, row.physical.pool=FALSE)

SPAS.print.model(mod2)

## -----------------------------------------------------------------------------
mod3 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling to single row",
                           row.pool.in=c(1,1,1), col.pool.in=1:3)
SPAS.print.model(mod3)

## -----------------------------------------------------------------------------
# do physical complete pooling 
mod4 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling all rows and last two colum ns",
                           row.pool.in=c(1,1,1), col.pool.in=c(1,1,3))
SPAS.print.model(mod4)

## -----------------------------------------------------------------------------
# do physical complete pooling 
mod5 <- SPAS.fit.model(test.data,
                           model.id="Physical complete pooling",
                           row.pool.in=c(1,1,1), col.pool.in=c(1,1,1))
SPAS.print.model(mod5)

## -----------------------------------------------------------------------------
model.list <- mget( ls()[grepl("^mod.$",ls())])
names(model.list)

report <- plyr::ldply(model.list, function(x){
   #browser()
   data.frame(#version=x$version,
              date   = as.Date(x$date),
              model.id         = x$model.info$model.id,
              s.a.pool         =-1+nrow(x$fit.setup$pooldata),
              t.p.pool         =-1+ncol(x$fit.setup$pooldata),
              logL.cond        = x$model.info$logL.cond,
              np               = x$model.info$np,
              AICc             = x$model.info$AICc,
              gof.chisq        = round(x$gof$chisq,1),
              gof.df           = x$gof$chisq.df,
              gof.p            = round(x$gof$chisq.p,3),
              Nhat             = round(x$est$real$N),
              Nhat.se          = round(x$se $real$N))
  
})
report

# 2020-01-01 CJS Now using TMB for optimization
#
#' Message to display when package is loaded
#' 
#' @importFrom utils packageVersion
#' @keywords internal
#'

.onAttach <- function(libname,pkgname){

  packageStartupMessage("***** SPAS: Stratified Petersen Analysis System - Version ", utils::packageVersion("SPAS"), " ***** \n\n",
        "      Help available with  help(package='SPAS') \n",
        '      Several vignettes are available. See browseVignettes(package="SPAS") \n\n')
}


# SEE http://r-pkgs.had.co.nz/src.html on why you need the following
.onUnload <- function (libpath) {
  library.dynam.unload("SPAS", libpath)
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
  
# define reporting and removal functions
get.models <- function(pattern){
    # get the list of models according to the pattern specified
    #browser()
    model.list <- mget( ls(envir=globalenv())[grepl(pattern,ls(envir=globalenv()))], envir=globalenv())
    model.list
}


make.report <- function(model.list){
   # make a little report
   report <- plyr::ldply(model.list, function(x){
      #browser()
      data.frame(#version=x$version,
         date   = as.Date(x$date),
         model.id         = x$model.info$model.id,
         s.a.pool         =-1+nrow(x$fit.setup$pooldata),
         t.p.pool         =-1+ncol(x$fit.setup$pooldata),
         logL.cond        = x$model.info$logL.cond,
         np               = x$model.info$np,
         AICc             = x$model.info$AICc,
         gof.chisq        = round(x$gof$chisq,1),
         gof.df           = x$gof$chisq.df,
         gof.p            = round(x$gof$chisq.p,3),
         Nhat             = round(x$est$real$N),
         Nhat.se          = round(x$se $real$N))
      
   })
   report
}

remove.models <- function(model.list){
    rm(list=names(model.list), envir=globalenv())
}


## -----------------------------------------------------------------------------

test.data.csv <- textConnection("
 160   ,   127   ,     72   ,     82   ,   3592
  24   ,    66   ,     13   ,     10   ,    532
7960   ,  9720   ,   6264   ,   7934   ,   0  ")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data

## -----------------------------------------------------------------------------
library(SPAS)
mod..1 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:2, col.pool.in=1:4)

SPAS.print.model(mod..1)

## -----------------------------------------------------------------------------
mod..2 <- SPAS.fit.model(test.data,
                           model.id="Pool last two columns",
                           row.pool.in=c(1,2), col.pool.in=c(1,2,34,34))

SPAS.print.model(mod..2)

## -----------------------------------------------------------------------------
mod..3 <- SPAS.fit.model(test.data,
                           model.id="Pool last two columns",
                           row.pool.in=c(1,2), col.pool.in=c(12,22,34,34))

SPAS.print.model(mod..3)

## ----echo=FALSE---------------------------------------------------------------
model.list <- get.models("^mod\\.\\.")
make.report(model.list)
remove.models(model.list)


## -----------------------------------------------------------------------------
mod..3 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling to single row",
                           row.pool.in=c(1,1), col.pool.in=1:4)
SPAS.print.model(mod..3)

## -----------------------------------------------------------------------------
mod..3a <- SPAS.fit.model(test.data,
                           model.id="Logical pooling to single row",
                           row.pool.in=c(1,1), col.pool.in=1:4, row.physical.pool=FALSE)
SPAS.print.model(mod..3a)

## -----------------------------------------------------------------------------
# do physcial complete pooling 
mod..4 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling all rows and last two columns",
                           row.pool.in=c(1,1), col.pool.in=c(12,12,34,34))
SPAS.print.model(mod..4)

## -----------------------------------------------------------------------------
# do physcial complete pooling 
mod..5 <- SPAS.fit.model(test.data,
                           model.id="Physical complete pooling",
                           row.pool.in=c(1,1), col.pool.in=c(1,1,1,1))
SPAS.print.model(mod..5)

## ----echo=FALSE---------------------------------------------------------------
model.list <- get.models("^mod\\.\\.")
make.report(model.list)
remove.models(model.list)

#' Helper functions for this package (logit, expit)
#' 
#' These functions are helper functions for the SPAS package and not normally accessed by the user
#' 
#' \code{logit} - compute the logit of a proportion
#' \code{expit} - compute a proportion from a logit
#' 
#' @keywords internal

# Miscellaneous functions for use in the modelling

# LOGIT amd EXPIT FUNCTIONs
logit <- function(p){
  # p is a proportion
  res <- log(p/(1-p))
  res <- pmax(-20, pmin(20, res)) # restrict the logit between -20 + 20
  return(res)
}

expit <- function(theta){ #antilogit,i.e. from logit to p scale\
  # theta is a logit
  res <- 1/(1+exp(-theta))
  return(res)
}

#' Autopooling a Stratified-Petersen (SP) data set.
#' This function applies pooling rules to pool a SPAS dataset to meeting minimum sparsity requirements
#' .
#' @template param.rawdata 
#' @template param.autopool.settings
#' 
#' @details
#' In many cases, the stratified set of releases and recapture is too sparse (many zeroes) or count
#' are very small. Pooling rows and columns may be needed.
#' 
#' Data needs to be pooled both row wise and column wise if the data are sparse 
#' to avoid singularities in the fit.  
#' This function automates pooling rows or columns following Schwarz and Taylor (1998).
#' * All rows that have 0 releases are discarded
#' * All columns that have 0 recaptures of tagged fish and 0 fish inspected are discarded
#' * Starting at the first row and working forwards in time,
#'   and then working from the final row and working backwards in time, 
#'.  rows are pooled until a minimum of \code{min.released} are released.
#'   An alternating pooling (from the top, from the bottom, from the top, etc) is used
#' * Starting at the first column and working forwards in time,
#'.   and then working from the final column and working backwards in time, 
#'    columns are pooled until a minimum of \code{min.inspected} are inspected.
#'    An alternating pooling (from the left, from the right, from the left, etc) is used.
#' * If the sum of the total recaptures from released fish is <= \code{min.recaps}, then all rows are pooled 
#' (which reduces to a Chapman estimator)
#' 
#' 
#' @return A list with a suggest pooling.
#' 
#' @export SPAS.autopool
#' @import msm MASS stats plyr TMB 
#' @importFrom Matrix bdiag 
#' @examples 
#' conne.data.csv <- textConnection("
#' 9  ,    21  ,     0  ,    0  ,    0  ,    0  ,   171
#' 0  ,   101  ,    22  ,    1  ,    0  ,    0  ,   763
#' 0  ,     0  ,   128  ,   49  ,    0  ,    0  ,   934
#' 0  ,     0  ,     0  ,   48  ,   12  ,    0  ,   434
#' 0  ,     0  ,     0  ,    0  ,    7  ,    0  ,    49
#' 0  ,     0  ,     0  ,    0  ,    0  ,    0  ,     4
#' 351,  2736  ,  3847  , 1818  ,  543  ,   191 ,     0")
#' conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
#' close(conne.data.csv)
#'  
#' SPAS.autopool(conne.data)
#' @md


SPAS.autopool<- function( rawdata, 
                          min.released  =100,
                          min.inspected = 50,
                          min.recaps    = 50,
                          min.rows=1, min.cols=1){

# Error checking of input values.
   # is Input data ok?
   if( !is.matrix(rawdata))stop("Input raw data must be a matrix")
   if( nrow(rawdata) < 2)  stop("Input raw data needs at least 2 rows")
   if( ncol(rawdata) < 2)  stop("Input raw data needs at least 2 columns")
   if( !is.numeric(rawdata))stop("Input raw data must be numeric")
  
   check.numeric(min.rows, min.value=1, max.value=nrow(rawdata)-1, req.length=1, check.whole=TRUE)
   check.numeric(min.cols, min.value=1, max.value=ncol(rawdata)-1, req.length=1, check.whole=TRUE)
   
   if(min.rows > min.cols)stop("Min rows must be <= min.cols")

   check.numeric(min.released,  min.value=1, req.length=1, check.whole=TRUE)
   check.numeric(min.inspected, min.value=1, req.length=1, check.whole=TRUE)
   check.numeric(min.recaps,    min.value=1, req.length=1, check.whole=TRUE)
  
   if(min.rows>1 | min.cols>1)warning("**** minimum rows or columns not yet implemented.****") 
   
   spas.mat <- rawdata
   
   s <- nrow(spas.mat) - 1
   t <- ncol(spas.mat) - 1
   
   #browser()
   # get rid of rows that have 0 releases
   select <- apply(spas.mat,1,sum, na.rm=TRUE)>0
   spas.red  <- spas.mat[select,]
   # get rid of columns that are all 0
   select <- apply(spas.red,2,sum, na.rm=TRUE)>0
   spas.red  <- spas.red[,select]
   
   # recompute s and t
   s.red <- nrow(spas.red)-1
   t.red <- ncol(spas.red)-1
   
   #browser()
   # pool columns until you get at least xxxx recoveries in a stratum and you hit the minimum number of rows etc
   # we will will work from the outside in
   col.pool <- 1:t.red
   col.sum  <- colSums(spas.red)
   left.last.col <- 1
   left.cum.sum  <- 0
   right.last.col <- t.red
   right.cum.sum  <- 0
   left <- 1
   right <- t.red
   min.count <- min.inspected
   while(left < right){
     col.pool[left] <- left.last.col
     left.cum.sum <- left.cum.sum + col.sum[left]
     if(left.cum.sum > min.count){
       left.last.col <- left+1
       left.cum.sum  <- 0
     }
     left <- left + 1
     
     col.pool[right] <- right.last.col
     right.cum.sum <- right.cum.sum + col.sum[right]
     if(right.cum.sum > min.count){
       right.last.col <- right-1
       right.cum.sum  <- 0
     }
     right <- right - 1
   }
   
   # pool rows until you get at least xxx releases  in a stratum or at least min.rows
   # we will will work from the outside in
   
   row.pool <- 1:s.red
   row.sum  <- rowSums(spas.red)
   top.last.row <- 1
   top.cum.sum  <- 0
   bot.last.row <- s.red
   bot.cum.sum  <- 0
   top <- 1
   bot <- s.red
   min.count <- min.released
   while(top < bot){
     row.pool[top] <- top.last.row
     top.cum.sum <- top.cum.sum + row.sum[top]
     if(top.cum.sum > min.count){
       top.last.row <- top+1
       top.cum.sum  <- 0
     }
     top <- top + 1
     
     row.pool[bot] <- bot.last.row
     bot.cum.sum <- bot.cum.sum + row.sum[bot]
     if(bot.cum.sum > min.count){
       bot.last.row <- bot-1
       bot.cum.sum  <- 0
     }
     bot <- bot - 1
   }
   
   # check the total recaptures
   if( sum(spas.mat[1:s, 1:t])<= 50){
     row.pool[] <- 1
   }  
   
   # return the original, reduced, and pooling vectors
   list(rawdata = rawdata,
        reddata = spas.red,
        row.pool= row.pool,
        col.pool= col.pool,
        summary = cbind(rbind(spas.red, c(col.pool, NA)), c(row.pool,NA,NA))
   )

}

#' Extract the estimates into a simpler form from a Stratified-Petersen fit
#' 
#' This function is normally used internally and not by users of the package.
#' 
#' @param est.red Vector of parameter estimates
#' @param rowDM Design matrix for covariates for row parameters (after pooling). Not currently implemented so
#'   this is usually an identity matrix.
#' @param colDM Design matrix for covariates for column parameters (after pooling). Not currently implemented so
#'   this is usually an identity matrix.
#' @param thetaDM Design matrix for covariates for movement parameters (after pooling). Not currently implemented so
#'   this is usually an identity matrix.
#' @param conditional Indicates if a conditional (Poisson model) or full likelihood (Multinomial) fit was done.
#'
#' @return 
#'   Returns a list with entries 
#' \itemize{
#'   \item theta The modeled number of animals released in row stratum i and recovered in column stratum j.
#'   \item cap. The estimated probability of tagging in row stratum i.
#'   \item psi. The estimated number of animals never seen after tagging for each row stratum.
#'   \item N.   The estimated population size.
#' }
#' @keywords internal




# SPAS.extract.par.est
# Extracts and creates the full estimates from the reduced parameter vector

# Input : 
#     Reduced parameter vector (in order)
#        theta.red.star  (on log scale)
#        cap.red.star    (on logit scale)
#        psi.red.star    (on log scale)
#        logN

SPAS.extract.par.est <- function(est.red,rowDM,colDM,thetaDM, conditional=TRUE){   
# extract the parameter estimates and create the parameter list
# if conditional=TRUE, then the N parameter does not exist
#
# first extact the reduced estimates, i.e. what is actually optimized

   n.theta.red.star <- ncol(thetaDM)
   theta.red.star <- est.red[1:n.theta.red.star]
   
   n.cap.red.star <- ncol(rowDM)
   cap.red.star   <- est.red[ (n.theta.red.star+1) : (n.theta.red.star+n.cap.red.star) ]
   
   n.psi.red.star <- nrow(rowDM) # not possible to put constraints on psi
   psi.red.star   <- est.red[(n.theta.red.star+n.cap.red.star+1)  : (n.theta.red.star+n.cap.red.star+n.psi.red.star)]
   if(!conditional) logN           <- est.red[(n.theta.red.star+n.cap.red.star+n.psi.red.star+1)]
   
#  expand the reduced estimates to the full  estimates using appropriate design matrices
   theta  <- thetaDM %*% theta.red.star
   cap    <- rowDM   %*% cap.red.star
   psi    <- psi.red.star  # no design matrices for psi
   
#  convert from logit to anti-logit, etc
   est <- NULL
   est$theta <- matrix(exp(theta), nrow=nrow(rowDM))
   est$psi   <- as.vector(exp(psi))
   est$cap   <- as.vector(expit(-cap)) # recall that estimates are log( (1-p)/p) rather than log( p/(1-p))
   if(!conditional)est$N     <- exp(logN)
   
   return(est)
} 

#' Score, likelihood, and related functions for fitting Stratified Petersen model
#' 
#' These functions are used internally to fit the Stratified Petersen models and are not normally used.
#' Most of these functions are for the legacy version that does not use TMB and so likely can be eventually deleted.
#' @keywords internal

# Likelihood (and related) functions for the Open SPAS models.

# SPAS.likelihood.star.DM.O(init.est, rowDM, colDM, thetaDM, rawdata, returnnegll=TRUE)
# SPAS.extract.par.est.O (res$par,rowDM,colDM,thetaDM, conditional=TRUE)
# SPAS.extract.par.est.O (init.est,rowDM,colDM,thetaDM, conditional=TRUE)

SPAS.likelihood.star.DM = function(Est.Star,rowDM,colDM,thetaDM,rawdata, returnnegll=FALSE, conditional=TRUE, fixed.N=0, mytrace=FALSE){
   #print(Est.Star)
   est <- SPAS.extract.par.est(Est.Star,rowDM,colDM,thetaDM, conditional=conditional)
   #browser()
   res <- SPAS.log.likelihood(est,rawdata, returnnegll=returnnegll, conditional=conditional, fixed.N=fixed.N, mytrace=mytrace)
   return(res)
} 


SPAS.score.DM = function(Est.Star,rowDM,colDM,thetaDM,rawdata, returnnegll=TRUE, conditional=TRUE){
   # compute the score (first partial deritivatives) based on the current estimates, design matrices, etc
   # This is computed only for the conditional likelihood at the moment.
  
   # convert the single vector into the actual estimates
   est <- SPAS.extract.par.est(Est.Star,rowDM,colDM,thetaDM, conditional=TRUE)
   #browser()
   
   s = nrow(rawdata)-1
   t = ncol(rawdata)-1

   # m matrix - Marked Fishes from strata s recaptured in strata t
   m = rawdata[1:s,1:t, drop=FALSE]

   # Fish Unrecovered from s-th stratum 
   n.10 = rawdata[1:s,t+1]

   # Unmarked Fish from the t-th stratum 
   u = rawdata[s+1,1:t]

   # Fish tagged for the s-th stratum
   n = rowSums(m) + n.10
   
   # score w.r.t. to parameter for number released and recaptured - theta
   # dont forget that Est.Star has log(theta.red) where theta = exp( thetaDM*theta.red)
   #
   score.theta.red <- m - est$theta  + 
                      diag((1-est$cap)/est$cap,nrow=length(est$cap),ncol=length(est$cap)) %*% 
                      est$theta %*% 
                      diag(u / as.vector(as.vector((1-est$cap)/est$cap) %*% est$theta)-1, ncol=length(u),nrow=length(u))
   score.theta  <- as.vector(score.theta.red) %*% thetaDM
   
   # score w.r.t. parameter for capture probabilities- caps
   # dont forget that Est.Star has MINUS logit(cap) as the parameter 
   score.cap <- as.vector((1-est$cap)/est$cap) * rowSums( est$theta %*% diag(u/ as.vector(as.vector((1-est$cap)/est$cap) %*% est$theta)-1, nrow=length(u), ncol=length(u))      )
   score.cap <- as.vector(score.cap) %*% rowDM
   
   # score w.r.t. to parameter for number released  but never seen - log(psi)
   # dont forget the Est.Star has log(psi) recorded
   score.psi <- n.10 - est$psi
   
   res <- c(score.theta, score.cap, score.psi)
   # but we are minimizing the negative of the loglikelihood
   if(returnnegll) {res <- -res}
#   print(res)
   return(res)
} 




# Log Likelihood Function for Open SPAS models
# Input : Full Estimate list on probability and anti-log scale for N, Raw Data.
# Output: Value. 
SPAS.log.likelihood <- function(est,rawdata, returnnegll=FALSE, conditional=FALSE, fixed.N=0, mytrace=FALSE){
# est - list of real parameter values with separate entry for each parameter type
# rawdata - the raw data in standard format
# returnnegll - should the negative of the log-likelihood be returned (for optimization purposes)
# condtional - should the conditional log likelihood poisson approximation be used (Plante and Rivest L_p)
  #browser()
  s = nrow(rawdata)-1
  t = ncol(rawdata)-1

  # m matrix - Marked Fishes from strata s recaptured in strata t
  m = rawdata[1:s,1:t, drop=FALSE]

  # Fish Unrecovered from s-th stratum 
  n.10 = rawdata[1:s,t+1]

  # Unmarked Fish from the t-th stratum 
  u = rawdata[s+1,1:t]

  # total fish seen
  n = sum(rawdata)

  # Movement Matrix
  theta = est$theta

  # Capture Probabilities
  cap = as.vector(est$cap)

  # Fish never seen again 
  psi <- as.vector(est$psi)
  
  # N hat only if the unconditional estimator is to be used.
  if(!conditional) N = est$N
  if(fixed.N>0) N=fixed.N

  if(!conditional){
     # The unconditional loglikelihood based on the the multionomial
     part1 = lgamma(N+1)  - lgamma(N-n+1) - lgamma(n+1)   
     part2 = sum(m    *log( (theta + as.numeric(theta==0))/N))
     part3 = sum(n.10 *log( (psi + as.numeric(psi==0))/N)) 
     part4 = sum(u    *log( (((1-cap)/cap) %*% theta+ as.numeric(u==0))/N))
     part5 = (N-n)    *log(1- (sum(theta)+sum(psi)+sum((((1-cap)/cap) %*% theta )))/N)
     l.l.h = part1 + part2 + part3 + part4 + part5 #- N*log(N)
     if( is.nan(l.l.h))browser()
     #browser()
  }
  if(mytrace)print(est)
   if(conditional){
     # the conditional log likelihood based on L_p of Plante and Rivest
     # Notice that conditional log-likelihood does not have N has a parameter
     part2 = sum(m * log(theta + as.numeric(theta==0))) - sum(theta)
     part3 = sum( n.10*log( psi + as.numeric(psi==0))) - sum(psi)
     part4 = sum(u*log(((1-cap)/cap) %*% theta+ as.numeric(u==0))) - sum( ((1-cap)/cap) %*% theta)
     l.l.h =  part2 + part3 + part4
   }
   # Negative Log Likelihood ? 
   if(returnnegll){ l.l.h <- -l.l.h}
   #browser()
   return(l.l.h)
}

# SPAS.i.fisher.DM
# This function creates the Variance Covariance Matrix for the Open SPAS models
#    Note: The inverse hessian is calculated using SVD.
#
# Input : 
#      Star Estimate Vector, row Design Matrix, Col Design Matrix, theta Design Matrix,  Raw Data
# Output: List of the following;
#        vcv = Variance Covariance Matrix. 
#        u = U orth matrix (SVD)
#        v = V orth matrix (SVD)
#        d.sig = Inverse Singular Values (SVD)

SPAS.i.fisher.DM <- function(est.star,rowDM,colDM,thetaDM,rawdata, conditional=FALSE, svd.cutoff=.0001){
   hes = -numDeriv::hessian(SPAS.likelihood.star.DM, x=est.star, method = ("Richardson"),
                 rowDM=rowDM,colDM=colDM,thetaDM=thetaDM,rawdata=rawdata, conditional=conditional)
   hess.svd = svd(hes)
   hess.svd$d = 1/(hess.svd$d+(hess.svd$d<svd.cutoff)) * (hess.svd$d > svd.cutoff)    
   vcv <- hess.svd$u %*% diag(hess.svd$d) %*% t(hess.svd$v )        
   return(list(vcv=vcv,u = hess.svd$u,v = hess.svd$v, d.sig = diag(hess.svd$d)) )
}


SPAS.i.hess <- function(hess, svd.cutoff=.0001){
   # compute the inverse of a matrix using singular value decomposition
   # this is needed because zero entries in the movement matrix have an estimated theta==0
   hess.svd = svd(hess)
   hess.svd$d = 1/(hess.svd$d+(hess.svd$d<svd.cutoff)) * (hess.svd$d > svd.cutoff)    
   vcv <- hess.svd$u %*% diag(hess.svd$d) %*% t(hess.svd$v )        
   return(list(vcv=vcv,u = hess.svd$u,v = hess.svd$v, d.sig = diag(hess.svd$d)) )
} 


# Model AICc
#This function calculates the AICc for the Open SPAS models.
# Input : 
#      Star Estimate Vector, Row Design Matrix, Col Design Matrix, Design Matrix, Theta Design Matrix, 
#      Raw Data.
# Output: AICC Value. 

SPAS.AICc <- function(est.star, rowDM, colDM, thetaDM,rawdata){
   est <- SPAS.extract.par.est(est.star, rowDM, colDM, thetaDM)
   np = length(est.star)  # numer of parameters  
   AIC =  2*np - 2*SPAS.likelihood.star.DM(est.star,rowDM,colDM,thetaDM,rawdata, returnnegll=FALSE, conditional=TRUE) #before correction
   n = sum(rawdata) # total sample size  
   AICc = AIC + 2*np*(np+1)/(n-np-1)
   return(AICc)
} 

#' Fit a Stratified-Petersen (SP) model using  TMB.
#' 
#' This function fits a Stratified-Petersen (Plante, 1996) to data and specify which rows/columns of the data
#' should be pooled. The number of rows after pooling should be <= number of columns after
#' pooling
#' .
#' @param model.id Character string identifying the name of the model including any pooling..
#' @template param.rawdata 
#' @template param.autopool
#' @template param.autopool.settings
#' @template param.row.pool.in
#' @template param.row.physical.pool
#' @param theta.pool,CJSpool NOT YET IMPLEMENTED. DO NOT CHANGE.
#' @param optMethod What optimization method is used. Defaults is the nlminb() function..
#' @param optMethod.control Control parameters for optimization method. See the documentation on the different optimization methods for details.
#' @param svd.cutoff When finding the variance-covariance matrix, a singular value decomposition is used. This identifies the smallest 
#'   singular value to retain.
#' @param chisq.cutoff When finding a goodness of fit statistic using (obs-exp)^2/exp, all cell whose Exp < gof.cutoff are ignored
#'        to try and remove structural zero cells.
#' @return A list with many entries. Refer to the vignettes for more details.
#' @export
#' @import msm MASS stats plyr TMB 
#' @importFrom Matrix bdiag 
#' @importFrom utils packageVersion
#' @examples 
#' conne.data.csv <- textConnection("
#' 9  ,    21  ,     0  ,    0  ,    0  ,    0  ,   171
#' 0  ,   101  ,    22  ,    1  ,    0  ,    0  ,   763
#' 0  ,     0  ,   128  ,   49  ,    0  ,    0  ,   934
#' 0  ,     0  ,     0  ,   48  ,   12  ,    0  ,   434
#' 0  ,     0  ,     0  ,    0  ,    7  ,    0  ,    49
#' 0  ,     0  ,     0  ,    0  ,    0  ,    0  ,     4
#' 351,  2736  ,  3847  , 1818  ,  543  ,   191 ,     0")
#' conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
#' close(conne.data.csv)
#'  
#' mod1 <- SPAS.fit.model(conne.data, model.id="Pooling rows 1/2, 5/6; pooling columns 5/6",
#'                       row.pool.in=c("12","12","3","4","56","56"),
#'                       col.pool.in=c(1,2,3,4,56,56))
#' mod2 <- SPAS.fit.model(conne.data, model.id="Auto pool",
#'                       autopool=TRUE)


# Fit the OPEN SPAS model to the data.
# This does the physical pooling of the data prior to the fit
# We have left in code to eventually allow you to specify covariates for capture-probabilties etc, but
# this has not yet been implemented.

# The final matrix, after pooling must have s <= t.

SPAS.fit.model<- function(model.id='Stratified Petersen Estimator', 
                          rawdata, autopool=FALSE,
                          row.pool.in=NULL, col.pool.in=NULL, 
                          row.physical.pool=TRUE, theta.pool=FALSE, CJSpool=FALSE, 
                          optMethod=c("nlminb"), 
                          optMethod.control=list(maxit = 50000), 
                          svd.cutoff=.0001, chisq.cutoff=.1,
                          min.released  =100, # autopooling settings
                          min.inspected = 50,
                          min.recaps    = 50,
                          min.rows=1, min.cols=1){
# Fit the Open SPAS model given the data and pooling information on the rows and columns
#
# The Open models have no constraints on the movement into the observed strata for each release group
# This is the model fit by the windoze SPAS program.
#
# rawdata - (s+1) x (t+1) raw data matrix
# row.pool.in - vector length s where duplicate entries indicate row to pool
# col.pool.in - vector length t where duplicate entries indicate columns to pool
# pooltheta   - logical indicator if want to pool for theta (movement) parameters - not yet implemented
# CJSpool     - logical indicator if want to have equal movement parameters along diagonal - not yet implemented
#
  
# Error checking of input values.
   # is Input data ok?
   if( !is.matrix(rawdata))stop("Input raw data must be a matrix")
   if( nrow(rawdata) < 2)  stop("Input raw data needs at least 2 rows")
   if( ncol(rawdata) < 2)  stop("Input raw data needs at least 2 columns")
   if( !is.numeric(rawdata))stop("Input raw data must be numeric")
  
   if(!is.logical(autopool) || length(autopool) !=1)stop("Autopool must be logical and length 1")
   if( autopool){
      if( !is.null(row.pool.in)) stop("Cannot specify row pooling if autopool=TRUE")
      if( !is.null(col.pool.in)) stop("Cannot specify columh pooling if autopool=TRUE")
   }
  
   if(!autopool){
      if(length(row.pool.in) != (nrow(rawdata)-1))stop("Row pooling vector not right length - number of rows(rawdata) -1")
      if(length(col.pool.in) != (ncol(rawdata)-1))stop("Column pooling vector not right length - number of columns(rawdata) -1")
   }
  
   # check that row.phyiscal.pool is logical.
   if(!is.logical(row.physical.pool) | length(row.physical.pool)!= 1)stop("Row.physical.pool must be logical of length 1")
   if(theta.pool)stop("Theta pooling not yet implemented")
   if(CJSpool   )stop("CJSpooling not yet implemented")

   # check input noise
   #if(!is.numeric(sd.noise.init.est))stop("sd.noise.input.est must be numeric")
  
   # check optimization method
   if(!optMethod[1] %in% c("nlminb"))stop("Invalid optimization method specified")
  
   if(!is.numeric(svd.cutoff))stop("Svd.cutoff must be numeric")
   if( svd.cutoff > .0001)stop("svd.cutoff is too large")   
  
   if(!is.numeric(chisq.cutoff) | length(chisq.cutoff) != 1)stop("chisquare cutoff must be numeric and length 1")
   
   if(autopool){
      res <- SPAS.autopool(rawdata, 
                           min.released  = min.released,
                           min.inspected = min.inspected,
                           min.recaps    = min.recaps,
                           min.rows      = min.rows, 
                           min.cols      = min.cols)
      rawdata <- res$reddata
      row.pool.in <- res$row.pool
      col.pool.in <- res$col.pool
   }

   # Does final pooled matrix have s <= t?
   if(length(unique(row.pool.in)) > length(unique(col.pool.in)))stop("S must be <= T after pooling")
  
   RESULT <- NULL
   RESULT$version <- paste0("SPAS-R ", utils::packageVersion("SPAS"))
   RESULT$date    <- Sys.time()   # date run and start date
   
   RESULT$input <- list(model.id    =model.id,
                        rawdata     = rawdata,
                        row.pool.in = row.pool.in,
                        col.pool.in = col.pool.in,
                        row.physical.pool=row.physical.pool,
                        theta.pool  = theta.pool,
                        CJSpool     = CJSpool,
                        chisq.cutoff= chisq.cutoff)
  
   s = nrow(rawdata)-1 # The number of release   strata prior to pooling
   t = ncol(rawdata)-1 # The number of recapture strata prior to pooling
   
   
#  row pooling first
   pool.frame <- data.frame(pool=as.factor(row.pool.in))
   if(length(unique(row.pool.in)) >1)  rowDM <- model.matrix( ~-1 + pool, data=pool.frame)
   if(length(unique(row.pool.in))==1)  {
      rowDM <- matrix(1, nrow=s, ncol=1)
      colnames(rowDM) <- as.character(row.pool.in[1])
   }
   if(row.physical.pool){  # physical pooling for the rows
      pool.row    <- t(rowDM) %*% rawdata[1:s,] 
      pool.row    <- rbind(pool.row, rawdata[s+1,])
      row.names(pool.row)[nrow(pool.row)] <- paste(row.names(rawdata)[s+1],"",sep="")
   }
   if(! row.physical.pool){  # logical pooling for the rows. nothing is done to the data
      pool.row <- rawdata
      row.names(pool.row) <- c(paste("pool", row.pool.in, sep="."), "")
      row.names(pool.row)[nrow(pool.row)] <- paste(row.names(rawdata)[s+1],"",sep="")
   }
   
#  column pooling next
   pool.frame <- data.frame(pool=as.factor(col.pool.in))
   if(length(unique(col.pool.in))> 1) colDM <- model.matrix( ~-1 + pool, data=pool.frame)
   if(length(unique(col.pool.in))==1)  {
      colDM <- matrix(1, nrow=t, ncol=1)
      colnames(colDM) <- as.character(col.pool.in[1])
   }
   # physical pooling of the rows
   pooldata    <- pool.row[, 1:t] %*% colDM 
   pooldata <- cbind(pooldata, pool.row[,t+1] )
   colnames(pooldata)[ncol(pooldata)] <-paste(colnames(pool.row)[t+1],"",sep="")
 
   s = nrow(pooldata)-1 # The number of release   strata AFTER physical pooling
   t = ncol(pooldata)-1 # The number of recapture strata AFTER physical pooling

# get the design matrices. If you do physical pooling, these are simple identity matrices
   if(row.physical.pool){
      rowDM  = diag(1, nrow(pooldata)-1)
      colDM  = diag(1, ncol(pooldata)-1)
      thetaDM= diag(1, (nrow(pooldata)-1)*(ncol(pooldata)-1))
   }
   #browser()
   if(!row.physical.pool){ #logical pooling so create the design matrices for rows and/columns
     # rowDM and colDM were computed earlier
     thetaDM = diag(1, (nrow(pooldata)-1)*(ncol(pooldata)-1)) # not yet implemented
   }

   # compute the condition number of XX' there X = physical matrix 
   # after physical pooling but prior to logical pooling
   # a large kappa value indicate the rows are colinear and so the estimates are unstable
   RESULT$kappa <- kappa( pooldata[1:s, 1:t,drop=FALSE] %*% t(pooldata[1:s, 1:t,drop=FALSE]))
   
   # compute the kappa after logical pooling of the data
   logical.pooldata <-   t(rowDM) %*% pooldata[1:s, 1:t,drop=FALSE]
   RESULT$kappa.after.lp <- kappa( logical.pooldata %*% t(logical.pooldata))
   
   # compute the pooled Petersen using the Chapman modification (add 1) and the se
   n1 <- sum(pooldata[1:s,    ])
   m2 <- sum(pooldata[1:s, 1:t])
   n2 <- sum(pooldata[   , 1:t])
   RESULT$est$N.Chapman = (n1+1)*(n2+1)/(m2+1) - 1
   RESULT$vcv$N.Chapman = ((n1+1)*(n2+1)*(n1-m2)*(n2-m2)) / ((m2+1)^2 * (m2+2))
   RESULT$se $N.Chapman = sqrt(RESULT$vcv$N.Chapman)
   
   
   # Initial Estimates obtained using least squares
   init.est = SPAS.init.est(rawdata=pooldata,rowDM=rowDM,colDM=colDM,thetaDM=thetaDM)

   # refer to https://stackoverflow.com/questions/48627069/guidelines-for-including-tmb-c-code-in-an-r-package
   # on how to include a TMB file with an R package
   
   # set up for the optimization 
   #dyn.load(dynlib("SPAS"))

   #browser() 
   # Create the model
   tmb.model <- MakeADFun(data= list(n_10 = pooldata[1:s  , t+1],
                                 u    = pooldata[s+1  , 1:t],
                                 m    = pooldata[1:s  , 1:t, drop=FALSE],
                                 rowDM=rowDM),
                        parameters=list(
                                 beta_cap   =as.vector(init.est$beta_cap),
                                 beta_theta =matrix(init.est$beta_theta, nrow=s, ncol=t),
                                 beta_psi   =as.vector(init.est$beta_psi)),
                        DLL="SPAS"
                        )

   #browser()
   
   
   # set the lower and upper bounds on the estimates 
   # theta are optimized on log scale; cap on logit scale; psi on log scale
   l.b = -12  # Lower Bound for optimization routine
   u.b =  20  # Upper Bound for optimization routine

   RESULT$fit.setup <- list(pooldata=pooldata,
                            rowDM = rowDM,
                            colDM = colDM,
                            thetaDM = thetaDM,
                            init.est= init.est,
                            l.b     = l.b,
                            u.b     = u.b,
                            tmb.model=tmb.model)
   # First optimization of the conditional likelihood.
   # This gives us the estimates of everything but N
    if(optMethod[1]=='nlminb'){
       cat("Using nlminb to find conditional MLE\n")
       fit <- nlminb(tmb.model$par, tmb.model$fn, tmb.model$gr, tmb.model$he, lower=l.b, upper=u.b)
       cat("Convergence codes from nlminb ", fit$convergence, fit$message, "\n")
   }

   RESULT$conditional$res$optim.info <- fit # from conditional maximization
   
   # Second estimate N using a conditional estimator .
   cat("Finding conditional estimate of N\n")
   
   # Extract the various estimates of the parameters from the optimization routine
   RESULT$est$beta$all <- fit$par
   
   # The psi terms. No design matrix. Beta parameters on log-scale
   beta_psi_index <- grepl("beta_psi", names(fit$par))
   RESULT$est$beta$psi <- fit$par[ beta_psi_index ]
   RESULT$est$real$psi <- exp(RESULT$est$beta$psi)
   
   # The theta terms. We have a design matrix and beta parameters on log scale, but nothing implemented yet
   beta_theta_index <- grepl("beta_theta", names(fit$par))
   RESULT$est$beta$theta <- fit$par[ beta_theta_index]
   RESULT$est$real$theta <- as.vector(exp(thetaDM %*% RESULT$est$beta$theta))
 
   # the capture probabilities. We have a design matrix and beta terms on logit scale
   beta_cap_index <- grepl("beta_cap", names(fit$par))
   RESULT$est$beta$cap <- fit$par[ beta_cap_index]
   RESULT$est$real$cap <- expit( as.vector(rowDM %*% RESULT$est$beta$cap)) 
   
   # extract the vcv for the various parameters and the se on the beta and regular scale
   RESULT$vcv$beta$allinfo <- SPAS.i.hess(tmb.model$he    ( fit$par))
   RESULT$vcv$beta$all     <- RESULT$vcv$beta$allinfo$vcv
   RESULT$vcv$beta$psi     <- RESULT$vcv$beta$all[ beta_psi_index,   beta_psi_index]
   RESULT$vcv$beta$cap     <- RESULT$vcv$beta$all[ beta_cap_index,   beta_cap_index]
   RESULT$vcv$beta$theta   <- RESULT$vcv$beta$all[ beta_theta_index, beta_theta_index]
   
   
   # Get the beta estimates after expanding by the design matrix
   # Note that the ordering of the parameters is determined by the .cpp file. So if that changes, you need to update this code.
   # (a) Go from the reduced estimates to the expanded estimates but still on the logit or log scale using the design matrices
   X <- bdiag(rowDM,                    # the logit(prob.capture) parameters
              thetaDM,                  # the log(theta) parameters
               diag(1,s)                # the log(psi) parameters
              ) 
   X <- as.matrix(X)   # sometime need to convert from sparse representation
   #browser()
   RESULT$est$beta$all.expanded <-  as.vector(X %*% RESULT$est$beta$all)
   RESULT$vcv$beta$all.expanded <-  X %*% RESULT$vcv$beta$all %*% t(X)

   # Now convert from the logit(), log() and log() scale to the real scale and find the vcv and se.
   # We use the deltamethod() function in the msm package
   # and so need to create extressions to do the delta method
   RESULT$est$real$all.expanded  <-c( RESULT$est$real$cap,   
                              RESULT$est$real$theta,
                              RESULT$est$real$psi)
   # set up the transformations 
   backtrans <- c(paste("~1/(1+exp(-x",    1:s,         "))"    , sep=""),
                  paste("~exp(x",      (s+1):(s+s*t),   ")"     , sep=""),
                  paste("~exp(x",  (s*t+s+1):(s*t+s+s),  ")"    , sep="")
                   )
   backtrans <- plyr::llply(backtrans, function(x){as.formula(x)})
   RESULT$vcv$real$all.expanded  <- msm::deltamethod(backtrans,  RESULT$est$beta$all.expanded,  RESULT$vcv$beta$all.expanded, ses=FALSE)
   RESULT$se $real$all.expanded  <- sqrt(diag( RESULT$vcv$real$all.expanded ))

   # extract the vcv of the real parameters and their se
   RESULT$vcv$real$cap    <- RESULT$vcv$real$all.expanded[        1:s  ,                 1:s      ,   drop=FALSE ]
   RESULT$vcv$real$theta  <- RESULT$vcv$real$all.expanded[    (s+1):(s+s*t)  ,       (s+1):(s+s*t),   drop=FALSE  ]
   RESULT$vcv$real$psi    <- RESULT$vcv$real$all.expanded[(s*t+s+1):(s*t+s+s)  , (s*t+s+1):(s*t+s+s), drop=FALSE ]
   
   RESULT$se$real$cap     <- sqrt(diag(RESULT$vcv$real$cap))
   RESULT$se$real$theta   <- sqrt(diag(RESULT$vcv$real$theta))
   RESULT$se$real$psi     <- sqrt(diag(RESULT$vcv$real$psi))
   
   # estimate N = total populaton size
   # Getting the variance of N (see equation 11 of Plante's paper)
   # Notice that we don't find the covariance of N with any of the other parameters

   RESULT$est$real$N  <- sum(pooldata, na.rm=TRUE) + tmb.model$report(fit$par)$neverseen
   part1 <- t(RESULT$est$real$psi) %*% diag(1/RESULT$est$real$cap^2, nrow=s) %*% 
            RESULT$vcv$real$cap %*%
            diag(1/RESULT$est$real$cap^2, nrow=s) %*% RESULT$est$real$psi
   part2 <- sum( RESULT$est$real$psi * ((1-RESULT$est$real$cap)/RESULT$est$real$cap + ((1-RESULT$est$real$cap)/RESULT$est$real$cap)^2 ))
   RESULT$vcv$real$N     <- part1 + part2
   RESULT$se$real$N      <- sqrt(RESULT$vcv$real$N)


   # Third. We now evaluate the conditional likelihood to get full covariance matrix of ALL parameters
   logL.cond   = -tmb.model$fn(fit$par)
   np          = length(fit$par)
   AIC =  2*np - 2*logL.cond #before correction
   n = sum(rawdata, na.rm=TRUE) # total sample size  
   AICc = AIC + 2*np*(np+1)/(n-np-1)

   RESULT$model.info <- list(model.id=model.id,
                             logL.cond   = logL.cond,
                             np          = np,       
                             AICc = AIC
                             ) 
   # Individual stratum estimates and their variances and set
   RESULT$est$real$N.stratum <- rowSums(pooldata)[1:s]/RESULT$est$real$cap 

   # Variances of individual stratum estimates (conditonal on number released)
   if( s >1){
     RESULT$vcv$real$N.stratum  <-   rowSums(pooldata)[1:s]^2 * (1/RESULT$est$real$cap^2)^2 * diag(RESULT$vcv$real$cap)
     RESULT$se $real$N.stratum  <-   sqrt(RESULT$vcv$real$N.stratum)
   }
   if( s==1){
     RESULT$vcv$real$N.stratum  <- RESULT$vcv$real$N
     RESULT$se $real$N.stratum  <- RESULT$se $real$N 
   }
   
   # Estimates of the expansion factors and their vcv
   RESULT$est$real$exp.factor     <- (1-RESULT$est$real$cap)/RESULT$est$real$cap
   RESULT$vcv$real$exp.factor     <- diag(1/RESULT$est$real$cap^2, nrow=s) %*% 
            RESULT$vcv$real$cap %*%
            diag(1/RESULT$est$real$cap^2, nrow=s)
   RESULT$se$real$exp.factor  <- sqrt(diag(RESULT$vcv$real$exp.factor))
 
   
   # Goodness-of-fit test. This is a simple chi-square test of observed vs. expected counts.
   # No pooling is done to deal with sparse counts, but a "cut off" of 0.1 is used to try
   # and eliminate cells with natural 0 counts (e.g. time stratification and t1 < t2)
   # The psi parameters are always a perfect fit and so no need to include them in the chi-square value
   # the cell counts, or the number of fitted parameters
   #browser()
   # find the component due to the movement matrix (m vs theta)
   gof.obs1 <- as.vector(RESULT$fit.setup$pooldata[1:(-1+nrow(RESULT$fit.setup$pooldata)), 1:(-1+ncol(RESULT$fit.setup$pooldata))])
   gof.exp1 <- as.vector(RESULT$est$real$theta)
   select   <- gof.exp1>chisq.cutoff
   chisq.gof <- sum((gof.obs1[select] - gof.exp1[select])^2/gof.exp1[select])
   
   # for the total number unobserved in recovery strata
   gof.obs2 <- RESULT$fit.setup$pooldata[nrow(RESULT$fit.setup$pooldata),1:(-1+ncol(RESULT$fit.setup$pooldata)) ]
   gof.exp2 <- as.vector((1-RESULT$est$real$cap)/RESULT$est$real$cap) %*% matrix(RESULT$est$real$theta,nrow=s, ncol=t) 
   select   <- gof.exp2>chisq.cutoff
   chisq.gof <- chisq.gof + sum((gof.obs2[select] - gof.exp2[select])^2/gof.exp2[select])
   chisq.gof.df <-  length(RESULT$est$real$theta) + s + t-RESULT$model.info$np
   RESULT$gof$chisq <- chisq.gof
   RESULT$gof$chisq.df <- chisq.gof.df
   RESULT$gof$chisq.p  <- ifelse(chisq.gof.df>0, pchisq(chisq.gof, df=chisq.gof.df, lower.tail=FALSE), NA)

   return(RESULT)            
}





#--------------------------------------------------------------------------------------------------
# SPAS.init.est;
# The following function creates initial estimates for the optimization routine for the OPEN SPAS models.
# using least squares estimators

# Input: raw data,
#        rowDM is a design matrix used to constrain the capture probabilities.
#        colDM is a design matrix used to constrain the column parameters   
#        thetaDM is a design matrix used to contrain the movement parameters   
#      
# Ouput: a list with entries
#       theta* = log( reduced theta parameters), i.e. as input to the design matrix
#       cap*   = log((1-p)/p) for each row
#       psi*   = log(reduced psi parameter - animals never recaptured)

#
SPAS.init.est <- function(rawdata,rowDM,colDM,thetaDM) { 

   s = nrow(rawdata)-1  # The number of release   stratas 
   t = ncol(rawdata)-1  # The number of recapture stratas
    
   # Number tagged in each stratum
   tagged = rowSums(rawdata)[1:s]
   # Unmarked captured
   unmarked = rawdata[s+1,1:t]
   # Marked fish recaptured
   m = rawdata[1:s,1:t, drop=FALSE]
   
   
   # The initial N hat estimate is found using the pooled petersen (a biased estimator). 
   N.hat = sum(tagged)*sum(c(m,unmarked))/sum(m)   
   # The N hat estimate is now transformed to the log scale. 
   logN.hat = log(N.hat)
   
   # number of fish tagged but never recovered (psi)
   psi.hat <- tagged - rowSums(m)
   psi.star.hat <- log(psi.hat) # no design matrix for psi, psihat is usually equal to actual psi
   
   # use a least squares estimator for the log (1-cap)/p
   logit.cap.hat <- log( pmax(.01, LS(t(m),unmarked) ))
   logit.cap.hat <- rep(mean(logit.cap.hat), length(logit.cap.hat))
   
   # initial theta values prior to using design matrix
   theta.hat <- as.vector(log(m+.1))
  
   # Now to find the estimates used with the design matrices. These are least-squares estimates
   logit.cap.star.hat <- LS(rowDM, logit.cap.hat)
   theta.star.hat     <- LS(thetaDM, theta.hat)
       
   res<- list(beta_theta=theta.star.hat, 
              beta_cap  = logit.cap.star.hat, 
              beta_psi  =psi.star.hat)
   return(res)
}


#Input : a Matrix A and Vector B (Design Matrix)
#Output: Least Squares Solution
LS <- function(A,b){
   x = MASS::ginv(t(A) %*% A) %*% t(A) %*% b
   return(x)
}

---
title: "Testing for Pooled Petersen"
author: "Carl James Schwarz"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    number_sections: yes
    toc: yes
    md_extensions: [ 
      "-autolink_bare_uris" 
    ]
vignette: >
  %\VignetteIndexEntry{Testing for Pooled Petersen} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Testing if a Pooled Petersen is appropriate

It is often of interest to know if a simple Pooled Petersen estimator, i.e., complete
pooling over rows and columns,
is appropriate.

As noted in Schwarz and Taylor (1998), the Pooled Petersen is unbiased 
under many conditions, but the most common are:

* Homogeneity of tagging probabilities, i.e. the probability of a fish being tagged in the release stratum is equal across all release strata.
* Homogeneity of recapture probabilities, i.e. the probability of a fish being 
recaptured is equal across all recovery strata.
* Complete Mixing, i.e. tagged fish mix completely with untagged fish.
* Correlation between tagging and recovery probabilities is zero, i.e. while
probabilities are heterogeneous across fish, the two events are statistically 
independent.

We can examine the first of these conditions by examining the results of the
stratified analysis and the results of a (logical) row pooling over all release
strata.

# Fitting the models

## Reading in the data
This data were made available from the Canadian Department of
Fisheries and Oceans and represent release and recaptured
of female fish in the Lower Shuswap region.

```{r }

test.data.csv <- textConnection("
  86   ,     54   ,     39   ,   219
  76   ,     35   ,     45   ,   168
  24   ,     53   ,     73   ,   190
1039   ,   1148   ,   2009   ,   0")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data
```

We now fit several models

* A fully 3x3 stratified analysis
* Pooling over all rows using logical pooling
* Pooling over all rows using physical pooling
* Pooling over all rows and the last two columns using physical pooling
* Complete physical pooling over rows and columns (a classical Pooled Petersen)

## Fully 3x3 stratified analysis
```{r }
library(SPAS)
mod1 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:3, col.pool.in=1:3)

SPAS.print.model(mod1)
```

#3 Pooling over all rows using logical pooling
```{r }
mod2 <- SPAS.fit.model(test.data,
                           model.id="Logical pooling to single row",
                           row.pool.in=c(1,1,1), col.pool.in=1:3, row.physical.pool=FALSE)

SPAS.print.model(mod2)
```

## Pooling over all rows using physical pooling
```{r }
mod3 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling to single row",
                           row.pool.in=c(1,1,1), col.pool.in=1:3)
SPAS.print.model(mod3)
```

## Pooling over all rows and last two columns using physical pooling
```{r}
# do physical complete pooling 
mod4 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling all rows and last two colum ns",
                           row.pool.in=c(1,1,1), col.pool.in=c(1,1,3))
SPAS.print.model(mod4)
```

## Complete physical pooling (Pooled Petersen Estimator)

```{r }
# do physical complete pooling 
mod5 <- SPAS.fit.model(test.data,
                           model.id="Physical complete pooling",
                           row.pool.in=c(1,1,1), col.pool.in=c(1,1,1))
SPAS.print.model(mod5)
```


# Get the model objects fitted by TMB and create a report

```{r }
model.list <- mget( ls()[grepl("^mod.$",ls())])
names(model.list)

report <- plyr::ldply(model.list, function(x){
   #browser()
   data.frame(#version=x$version,
              date   = as.Date(x$date),
              model.id         = x$model.info$model.id,
              s.a.pool         =-1+nrow(x$fit.setup$pooldata),
              t.p.pool         =-1+ncol(x$fit.setup$pooldata),
              logL.cond        = x$model.info$logL.cond,
              np               = x$model.info$np,
              AICc             = x$model.info$AICc,
              gof.chisq        = round(x$gof$chisq,1),
              gof.df           = x$gof$chisq.df,
              gof.p            = round(x$gof$chisq.p,3),
              Nhat             = round(x$est$real$N),
              Nhat.se          = round(x$se $real$N))
  
})
report
```

The AIC should be compared ONLY for the first two models because they are based on the same set of data.
**You cannot compare models that differ in the physical pooling**

In this case, there is good evidence that the Pooled Petersen is too coarse because the goodness
of fit statistic  for the second model is very large (with a corresponding small goodness-of-fit p-value).
Similarly, the AIC indicates that the model is 3x3 stratification (first model) is preferable to the model with 
complete row pooling (second model).

Notice that the estimates of the population size are identical under logical or physical row pooling (models 2 and 3).
And how you pool columns (models 3, 4, 5) but assuming that the number of rows (after logical or physical pooling as long
the number of rows
is not larger than the number of columns) does not affect the population size estimate (or standard error).


# References
Darroch, J. N. (1961). The two-sample capture-recapture census when tagging and sampling are stratified. Biometrika, 48, 241–260.
https://www.jstor.org/stable/2332748

Plante, N., L.-P Rivest, and G. Tremblay. (1988). Stratified Capture-Recapture Estimation of the Size of a Closed Population. Biometrics 54, 47-60.
https://www.jstor.org/stable/2533994

Schwarz, C. J., & Taylor, C. G. (1998). The use of the stratified-Petersen estimator in fisheries management with an illustration of estimating the number of pink salmon (Oncorhynchus gorbuscha) that return to spawn in the Fraser River. Canadian Journal of Fisheries and Aquatic Sciences, 55, 281–296.
https://doi.org/10.1139/f97-238

---
title: "Things that can go wrong"
author: "Carl James Schwarz"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    number_sections: yes
    toc: yes
    md_extensions: [ 
      "-autolink_bare_uris" 
    ]
vignette: >
  %\VignetteIndexEntry{Things that can go wrong} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

library(SPAS)  

```

# Things that can go wrong.
There are several types of problem that can occur that make the use of 
SPAS problematic.

## Rows are proportional
If the row in the movement matrix are proportional, then there an infinite
number of solutions. Note it is the singularity of the movement matrix 
that is problematic; the number of tags releases and not recovered (last column of
the input data) can be proportional without affecting the fit.

The model may converge to different solutions depending on your set up.
There is no automatic "flagging" of this situation and you need
to be vigilant. A diagnostic tool is the condition number of
$XX'$ where $X$ is the movement matrix. If the rows are exactly 
proportional (or more generally if the rows are colinear) the
condition number will be $+\infty$. 

The solution is to pool the row(s) that are proportional.

Here are some examples:

```{r }

test.data.csv <- textConnection("
 160   ,   120   ,     72   ,     82   ,   3592
  80   ,    60   ,     36   ,     41   ,    532
7960   ,  9720   ,   6264   ,   7934   ,   0  ")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data

mod..1 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:2, col.pool.in=1:4)

SPAS.print.model(mod..1)
```

In this case, it appears that the model has converged, but this is ONE of many possible 
solutions and cannot be relied upon.

The condition number is very large (!) as expected and shown in the above output:
```{r }
# Compute the condition number of XX'

XX <- test.data[1:2, 1:4] %*% t(test.data[1:2, 1:4])
XX
cat("\n\nCondition number is\n")
kappa(XX)
```

We will either physically or logically pool rows:
```{r echo=TRUE}
mod..2 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=c(1,1), col.pool.in=1:4, 
                       row.physical.pool=FALSE)

SPAS.print.model(mod..2)
```

This has solved the colinearity problem.

## Rows are approximately proportional
Of course with real data, it is highly unlikely that the rows in the
recovery matrix will be exactly proportional. 

The model may converge to different solutions depending on your set up.
There is no automatic "flagging" of this situation and you need
to be vigilant. The condition number of $XX'$ may be useful.

The solution is to pool the row(s) that are proportional.

Here are some examples where the entries in row 2 are modified
slightly to make the rows only approximately proportional

```{r }

test.data.csv <- textConnection("
 160   ,   120   ,     72   ,     82   ,   3592
  75   ,    62   ,     38   ,     35   ,    532
7960   ,  9720   ,   6264   ,   7934   ,   0  ")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data

mod..2 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:2, col.pool.in=1:4)

SPAS.print.model(mod..2)

# Compute the condition number of XX'

XX <- test.data[1:2, 1:4] %*% t(test.data[1:2, 1:4])
XX
cat("\n\nCondition number is\n")
kappa(XX)
```

Now there is only one solution, but the estimate is very sensitive
to small changes in the data. Notice that the condition number
is still very large.


# Columns that are all zero
In theory this should have no influence on the fit (see the 
section on pooling columns). 

```{r }

test.data.csv <- textConnection("
 160   ,   120   ,     72   ,     82   ,   0, 3592
 100   ,    45   ,     39   ,     90   ,   0,  532
7960   ,  9720   ,   6264   ,   7934   ,   0,    0  ")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data

mod..3 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:2, col.pool.in=1:5)

SPAS.print.model(mod..3)

XX <- test.data[1:2, 1:5] %*% t(test.data[1:2, 1:5])
XX
cat("\n\nCondition number is\n")
kappa(XX)
```

Notice that the column of 0's does not affect the fit
and has no impact on the condition number of $XX'$.


# References
Darroch, J. N. (1961). The two-sample capture-recapture census when tagging and sampling are stratified. Biometrika, 48, 241–260.
https://www.jstor.org/stable/2332748

Plante, N., L.-P Rivest, and G. Tremblay. (1988). Stratified Capture-Recapture Estimation of the Size of a Closed Population. Biometrics 54, 47-60.
https://www.jstor.org/stable/2533994

Schwarz, C. J., & Taylor, C. G. (1998). The use of the stratified-Petersen estimator in fisheries management with an illustration of estimating the number of pink salmon (Oncorhynchus gorbuscha) that return to spawn in the Fraser River. Canadian Journal of Fisheries and Aquatic Sciences, 55, 281–296.
https://doi.org/10.1139/f97-238

---
title: "Conne River 1992 Data"
author: "Carl James Schwarz"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    toc: true
    number_sections: yes
    md_extensions: [ 
      "-autolink_bare_uris" 
    ]
vignette: >
  %\VignetteIndexEntry{Conne River 1992 Data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction
This analyzes a dataset from the Conne River, Newfoundland collected in 1992. Smolts are tagged and released at one location in the river.
A fence at a second station also captures fish. Data are stratified on a weekly level.

A stratified-Petersen estimator (Darroch, 1961; Plante et al. 1996) is found for the original data and some pooling of the rows/columns.

# Load the SPAS package
```{r libraries}
library(SPAS)
```
This will load the SPAS fitting functions and any associated packages needed by SPAS.

# Import the data.
The data  should be stored as an $s+1$ by $t+1$ (before pooling) matrix.
The $s \times t$ upper left matrix is the number of animals released in row stratum $i$ and recovered in
column stratum $j$. 
Row $s+1$ contains the total number of UNMARKED animals recovered in column stratum $j$.
Column $t+1$ contains the number of animals marked in each row stratum but not recovered in any column stratum.
The $rawdata[s+1, t+1]$ is not used and can be set to 0 or NA.
The sum of the entries in each of the first $s$ rows is then the number of animals marked in each row stratum.
The sum of the entries in each of the first $t$ columns is then the number of animals captured (marked and unmarked) in each column stratum.
The row/column names of the matrix may be set to identify the entries in the output.

Here is the raw data for the Conne River:
```{r loaddata}
conne.data.csv <- textConnection("
 149,  126,    0,    0,    0,    0,      1561
   0,  308,   65,    0,    0,    0,      1235
   0,    0,  161,   77,    0,    0,       884
   0,    0,    0,   67,    7,    0,       215
   0,    0,    0,    0,   17,    3,        71
   0,    0,    0,    0,    0,   13,        16
1895, 8503, 2184,  525,  155,  118,         0")
conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
```
A total of $149+126+1561=1836$ fish were tagged and released in week 1. Of these fish, $149$ were recovered down stream in column stratum 1;
126 were recovered in column stratum 2; and $1561$ were never seen again.
A total of $1895$ UNMARKED fish were recovered in column stratum 1.

You can add row and column names to the matrix which will used when the data are printed.

# Fitting the Stratified-Petersen model
The Stratified-Petersen model is fit to the above data object. The code to fit the model is:
```{r fit1,results="hide"}
mod1 <- SPAS.fit.model(conne.data,
                       model.id="No restrictions",
                       row.pool.in=1:6, col.pool.in=1:6)
```
The `row.pool.in` and `col.pool.in` inform the function on which rows or columns to pool before the analysis proceeds. 
Both parameters use a vector of codes (length $s$ for row pooing and length $t$ for column pooling) where rows/columns
are pooled that share the same value.

For example. `row.pool.in=c(1,2,3,4,5,6)` would imply that no rows are pooled, while
`row.pool.in=c('a','a','a','b','b','b')` would imply that the first three rows and last three rows are pooled.
The entries in the vector can be numeric or character; however, using character entries implies that the final
pooled matrix is displayed in the order of the character entries. I find that using entries such as `'123'` to represent
pooling rows 1, 2, and 3 to be easiest to use.

**The SPAS system only fits models were the number of rows after pooling is less than or equal to the number of columns after pooling.**

## Results from model 1 (no pooling).
The results of the model fit is a LARGE list. But the function `SPAS.print.model` produces a nice report
```{r mod1p}
SPAS.print.model(mod1)
```

The original data, the data after pooling, estimates and their standard errors are shown.
Here the stratified-Petersen estimate of the total number of smolts passing the at the first sampling station is
`r formatC(round(mod1$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod1$se$ real$N), format='f', digits=0, big.mark=',')`.


Each entry in the output is available in fitted model object. You will need to explore the
structure
```{r str1}
cat("Names of objects at highest level\n")
names(mod1)
cat("\n\nNames of estimates (both beta and real)\n")
names(mod1$est)
cat("\n\nNames of real estimates\n")
names(mod1$est$real)
```
The `N` entries refer to the  population size; the `N.stratum` entries refer to the individual stratum population sizes;
the `cap` entries refer to the estimated probability of capture in each row stratum;
the `exp.factor` entries refer to `(1-cap)/cap`, or the expansion factor for each row;
the `psi` entries refer to the number of animals tagged but never seen again (the right most column in the input data);
and the `theta` entries refer to the expected number of animals that were tagged in row stratum $i$ and recovered in column stratum $j$ (after pooling).

# Physically pooling some rows and columns
As noted by Darroch (1961), the stratified-Petersen will fail if the matrix of movements is close to singular. This often happens if two rows are proportional to each other. In this case, there is no unique MLE for the probability of capture in the two rows, and they should be pooled.
A detailed discussion of pooling is found in Schwarz and Taylor (1998).

Let us now pool the first two rows and the last two rows, but leave rows 3 and 4 alone.
Similar, let us pool the last two columns.

The code is
```{r fit2,results="hide"}
mod2 <- SPAS.fit.model(conne.data, model.id="Pooling some rows",
                       row.pool.in=c("12","12","3","4","56","56"),
                       col.pool.in=c(1,2,3,4,56,56))
```

Notice how we specify the pooling for rows and columns and the choice of entries for the two corresponding vectors.
I used character entries for the row pooling to ensure that the pooled matrix is sorted properly (see below);
the numeric entries for the columns is ok as is.

The results of the model fit is
```{r mod2p}
SPAS.print.model(mod2)
```

Here the stratified-Petersen estimate of the total number of smolts passing the at the first sampling station is
`r formatC(round(mod2$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod2$se$ real$N), format='f', digits=0, big.mark=',')`.
which is a slight reduction
from the unpooled estimates.

## Pooling to a single row and complete pooling
You can pool to a single row (and multiple columns) or a single row and single column (which is equivalent to the pooled Petersen estimator).
The code and output follow:

```{r mod3,results='hide'}
mod3 <- SPAS.fit.model(conne.data, model.id="A single row",
                       row.pool.in=rep(1, nrow(conne.data)-1),
                       col.pool.in=c(1,2,3,4,56,56))
```
```{r mod3p}
SPAS.print.model(mod3)
```

```{r mod4,results='hide'}
mod4 <- SPAS.fit.model(conne.data, model.id="Pooled Peteren",
                       row.pool.in=rep(1, nrow(conne.data)-1),
                       col.pool.in=rep(1, ncol(conne.data)-1))
```
```{r mod4p}
SPAS.print.model(mod4)
```
Both models have an estimated abundance of
`r formatC(round(mod3$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod3$se$ real$N), format='f', digits=0, big.mark=',')`.
which is a slight reduction
from the unpooled estimates.

## Comparing different pooling
**Unfortunately, because pooling actually changes the data (internally), it is not possible to
do likelihood ratio testing or to use AICc to compare physical different poolings.**


# References
Darroch, J. N. (1961). The two-sample capture-recapture census when tagging and sampling are stratified. Biometrika, 48, 241–260.
https://www.jstor.org/stable/2332748

Plante, N., L.-P Rivest, and G. Tremblay. (1988). Stratified Capture-Recapture Estimation of the Size of a Closed Population. Biometrics 54, 47-60.
https://www.jstor.org/stable/2533994

Schwarz, C. J., & Taylor, C. G. (1998). The use of the stratified-Petersen estimator in fisheries management with an illustration of estimating the number of pink salmon (Oncorhynchus gorbuscha) that return to spawn in the Fraser River. Canadian Journal of Fisheries and Aquatic Sciences, 55, 281–296.
https://doi.org/10.1139/f97-238

---
title: "Pooling Columns"
author: "Carl James Schwarz"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    number_sections: yes
    toc: yes
    md_extensions: [ 
      "-autolink_bare_uris" 
    ]
vignette: >
  %\VignetteIndexEntry{Pooling Columns} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
  
# define reporting and removal functions
get.models <- function(pattern){
    # get the list of models according to the pattern specified
    #browser()
    model.list <- mget( ls(envir=globalenv())[grepl(pattern,ls(envir=globalenv()))], envir=globalenv())
    model.list
}


make.report <- function(model.list){
   # make a little report
   report <- plyr::ldply(model.list, function(x){
      #browser()
      data.frame(#version=x$version,
         date   = as.Date(x$date),
         model.id         = x$model.info$model.id,
         s.a.pool         =-1+nrow(x$fit.setup$pooldata),
         t.p.pool         =-1+ncol(x$fit.setup$pooldata),
         logL.cond        = x$model.info$logL.cond,
         np               = x$model.info$np,
         AICc             = x$model.info$AICc,
         gof.chisq        = round(x$gof$chisq,1),
         gof.df           = x$gof$chisq.df,
         gof.p            = round(x$gof$chisq.p,3),
         Nhat             = round(x$est$real$N),
         Nhat.se          = round(x$se $real$N))
      
   })
   report
}

remove.models <- function(model.list){
    rm(list=names(model.list), envir=globalenv())
}

```

# Why it is not possible to compare different column poolings.

There are two issues that currently preclude a direct implementation of logical pooling for columns. 

First, unlike pooling rows where physical pooling is equivalent to equating the initial tagging probabilities, there is no equivalent rule for recovery probabilities for column pooling. 
In the current model, the expected values for cells counts are (Schwarz and Taylor) in the 2 x 3 case (i.e. $s \le t$):

Releases | Recovery stratum 1|Recovery stratum 2|Recovery stratum 3 
-----------------|---------------|--------------|--------------- 
Release stratum 1 |	$p_1 \theta_{11} r_1$|$p_1 \theta_{12} r_2$|$p_1 \theta_{13} r_3$  
Release stratum 2 |	$p_2 \theta_{21} r_1$|$p_2 \theta_{22} r_2$|$p_2 \theta_{23} r_3$  
Unmarked|$\sum{(1-p_i)\theta_{i1}r_1}$|$\sum{(1-p_i)\theta_{i2}r_2}$|$\sum{(1-p_i)\theta_{i3}r_3}$


where $p_i$ is the tagging probability in release stratum $i$; 
$r_j$ is the recovery probability in recovery stratum $j$; 
and $\theta_{ij}$ is the probability of moving from release stratum $i$ 
to recovery stratum $j$. 
In the case of $s < t$, the recovery probabilities are not separately identifiable because in any column, we can multiply $\theta_{ij}$ by a constant $k$ and divide $r_j$ by $k$ 
and get exactly the same expected values. 
Hence one can always force $r_i = r_j$ for any ($i$ and $j$) pair by appropriate choice 
of $k$ values for each column.

Second, in theory, you can always pool columns and NOT affect the fit. 
As an analogy, the SPAS model is (to the first order approximation) 
a regression problem, i.e. given a matrix of recoveries and data

Stratum |Recovery stratum 1 |Recovery stratum 2|Recovery stratum 3
------|-----------|-----------|------------
Release stratum 1	| $m_{11}$|$m_{12}$|$m_{13}$
Release stratum 2 | $m_{21}$|$m_{22}$|$m_{23}$
Unmarked|$u_1$|$u_2$|$u_2$
			
You want to find estimates of $\beta_1$ and $\beta_2$ such that
$$u_1≅\beta_1 m_{11}+\beta_2 m_{21}$$
$$u_2≅\beta_1 m_{12}+\beta_2 m_{22}$$
$$u_3≅\beta_1 m_{13}+\beta_2 m_{23}$$

Pooling columns 2 and 3 reduces this system of equations to:
$$u_1≅\beta_1 m_{11}+\beta_2 m_{21}$$
$$u_2+u_3≅\beta_1 (m_{12}+m_{13})+\beta_2 (m_{22} +m_{23})$$
which has the identical fit.

Like the regression analogy, pooling columns is like pooling two data points 
in a regression setting by adding the respective $X$ and $Y$ values. 
You will very similar same regression estimates, particularly if you do a weighted regression to account for the doubling of the variation when you add to points together.

So in theory, pooling columns should have negligible effect on the estimates of the population size.
So at the moment, I doubt that it is possible to implement logical pooling of columns and compare column pooling using AIC. 
It would be possible to implement logical pooling of columns but it solves an uninteresting problem of testing if the product of movement and recovery are identical for all release groups in the two columns which is seldom biologically plausible.



# A sample dataset
This sample data set was adopted from the Canadian Department of
Fisheries and Oceans and represent release and recaptured
of female fish in the Lower Shuswap region.

```{r }

test.data.csv <- textConnection("
 160   ,   127   ,     72   ,     82   ,   3592
  24   ,    66   ,     13   ,     10   ,    532
7960   ,  9720   ,   6264   ,   7934   ,   0  ")

test.data <- as.matrix(read.csv(test.data.csv, header=FALSE, strip.white=TRUE))
test.data
```

# Models with 2 rows and different number of columns
We now fit two models examining the impact of pooling columns
with different types of pooling rows.


## Full 2x4 stratified analysis
```{r }
library(SPAS)
mod..1 <- SPAS.fit.model(test.data,
                       model.id="No restrictions",
                       row.pool.in=1:2, col.pool.in=1:4)

SPAS.print.model(mod..1)
```

## Reducing to a 2x3 matrix by pooling columns 3 and 4.
```{r }
mod..2 <- SPAS.fit.model(test.data,
                           model.id="Pool last two columns",
                           row.pool.in=c(1,2), col.pool.in=c(1,2,34,34))

SPAS.print.model(mod..2)
```

## Reducing to a 2x2 matrix by pooling columns 1 and 2, and then 3 and 4.
```{r }
mod..3 <- SPAS.fit.model(test.data,
                           model.id="Pool last two columns",
                           row.pool.in=c(1,2), col.pool.in=c(12,22,34,34))

SPAS.print.model(mod..3)
```
## Comparing the estimates 
Notice that the population estimate and its standard error
is identical to the unpooled case. You cannot compare the AICc values
because the data set has changed between the two fits.

```{r echo=FALSE}
model.list <- get.models("^mod\\.\\.")
make.report(model.list)
remove.models(model.list)

```

# Comparing results with 1 row and different number of columns

## Pooling over all rows using physical pooling
```{r }
mod..3 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling to single row",
                           row.pool.in=c(1,1), col.pool.in=1:4)
SPAS.print.model(mod..3)
```

## Pooling over all rows using logical pooling
```{r }
mod..3a <- SPAS.fit.model(test.data,
                           model.id="Logical pooling to single row",
                           row.pool.in=c(1,1), col.pool.in=1:4, row.physical.pool=FALSE)
SPAS.print.model(mod..3a)
```
## Pooling over all rows and last two columns using physical pooling
```{r}
# do physcial complete pooling 
mod..4 <- SPAS.fit.model(test.data,
                           model.id="Physical pooling all rows and last two columns",
                           row.pool.in=c(1,1), col.pool.in=c(12,12,34,34))
SPAS.print.model(mod..4)
```

## Complete physical pooling (Pooled Petersen Estimator)

```{r }
# do physcial complete pooling 
mod..5 <- SPAS.fit.model(test.data,
                           model.id="Physical complete pooling",
                           row.pool.in=c(1,1), col.pool.in=c(1,1,1,1))
SPAS.print.model(mod..5)
```


## Comparing the estimates of abundance
```{r echo=FALSE}
model.list <- get.models("^mod\\.\\.")
make.report(model.list)
remove.models(model.list)

```


Notice that the estimates of the population size are identical 
under logical or physical row pooling.


# References
Darroch, J. N. (1961). The two-sample capture-recapture census when tagging and sampling are stratified. Biometrika, 48, 241–260.
https://www.jstor.org/stable/2332748

Plante, N., L.-P Rivest, and G. Tremblay. (1988). Stratified Capture-Recapture Estimation of the Size of a Closed Population. Biometrics 54, 47-60.
https://www.jstor.org/stable/2533994

Schwarz, C. J., & Taylor, C. G. (1998). The use of the stratified-Petersen estimator in fisheries management with an illustration of estimating the number of pink salmon (Oncorhynchus gorbuscha) that return to spawn in the Fraser River. Canadian Journal of Fisheries and Aquatic Sciences, 55, 281–296.
https://doi.org/10.1139/f97-238

---
title: "Automatic Pooling"
author: "Carl James Schwarz"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    number_sections: yes
    toc: yes
    md_extensions: [ 
      "-autolink_bare_uris" 
    ]
vignette: >
  %\VignetteIndexEntry{Automatic Pooling} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

Several vignettes explored how to pool rows and/or columns to deal with the near singularity
of the recapture matrix. A new function has been introduced *SPAS.autopool()* that attempts
to automate this process.

The automatic pooling algorithm follows recommendations in Schwarz and Taylor (1998)
and proceeds as follows:

- All rows that have 0 releases are discarded
- All columns that have 0 recaptures of tagged fish and 0 fish inspected are discarded
- Starting at the first row and working forwards in time,
and then working from the final row and working backwards in time,
rows are pooled until a minimum of *min.released* are released.
An alternating pooling (from the top, from the bottom, from the top, etc) is used
- Starting at the first column and working forwards in time,
and then working from the final column and working backwards in time,
columns are pooled until a minimum of *min.inspected* are inspected.
An alternating pooling (from the left, from the right, from the left, etc) is used.
- If the sum of the total recaptures from released fish is <= *min.recaps*, then all rows are pooled
(which reduces to a Chapman estimator)

The values of *min.released*,*min.inspected*,*min.recaps* can be passed as
arguments to the function.

The function returns a suggested pooling vector for the rows and columns and a reduced
data matrix if there are rows that are all zero or columns that are all zero.

The *SPAS.fit.model()* function also now has a *autopool* argument where an automated
pooling will be attempted rather than you passing the pooling vectors.

# Example. 

The example from the Harrison River will be used:

## Load the SPAS package

```{r libraries}
library(SPAS)
```
This will load the SPAS fitting functions and any associated packages needed by SPAS.

## Import the data.

The data  should be stored as an $s+1$ by $t+1$ (before pooling) matrix.
The $s \times t$ upper left matrix is the number of animals released in row stratum $i$ and recovered in
column stratum $j$. 
Row $s+1$ contains the total number of UNMARKED animals recovered in column stratum $j$.
Column $t+1$ contains the number of animals marked in each row stratum but not recovered in any column stratum.
The $rawdata[s+1, t+1]$ is not used and can be set to 0 or NA.
The sum of the entries in each of the first $s$ rows is then the number of animals marked in each row stratum.
The sum of the entries in each of the first $t$ columns is then the number of animals captured (marked and unmarked) in each column stratum.
The row/column names of the matrix may be set to identify the entries in the output.

Here is the raw data for the Harrison River. Notice that very small number of releases and recoveries in week 6. 

```{r loaddata}
harrison.2011.chinook.F.csv <- textConnection("
  4   ,      2   ,      1   ,     1   ,     0   ,     0   ,   130
 12   ,      7   ,     14   ,     1   ,     3   ,     0   ,   330
  7   ,     11   ,     41   ,     9   ,     1   ,     1   ,   790
  1   ,     13   ,     40   ,    12   ,     9   ,     1   ,   667
  0   ,      1   ,      8   ,     8   ,     3   ,     0   ,   309
  0   ,      0   ,      0   ,     0   ,     0   ,     1   ,    65
744   ,   1187   ,   2136   ,   951   ,   608   ,   127   ,     0")

har.data <- as.matrix(read.csv(harrison.2011.chinook.F.csv, header=FALSE))
har.data
```

A total of `r sum(har.data[1,])` fish were tagged and released in week 1. Of these fish, `r har.data[1,1]` were recovered down stream in column stratum 1;
`r har.data[1,2]` were recovered in column stratum 2; and `r har.data[1,7]` were never seen again.
A total of `r har.data[7,1]` UNMARKED fish were recovered in column stratum 1.

You can add row and column names to the matrix which will used when the data are printed.

## Automate Pooling

We invoke the *SPAS.autopool()* function and look at the output:

```{r}
res <- SPAS.autopool(har.data)
res

```

There were no rows or columns that were all zero, so the 
reduced data (*res*) is the same as the original data.

The suggested pooling vector for columns indicates no pooling was done (all entries are the same).
The suggested pooling vector for rows, suggests rows 5 and 6 be pooled as indicated by the
duplicate entries of *6* in the pooling vector.

Finally, the reduced data and suggest pooling vectors are presented in one display.

## Fitting allowing for automated pooling.

We can use the *autopool* argument in the *SPAS.fit.model* to do automated pooling
prior to the fit.

```{r}
mod..1 <- SPAS.fit.model(har.data,
                       model.id="Automated pooling",
                       autopool=TRUE)
SPAS.print.model(mod..1)
```

The automated pooling combined the last two rows, but the fit is not entire satisfactory
because of the high condition number on *X'X*.

Some further bespoke pooling is likely necessary.


# References

Schwarz, C. J., & Taylor, C. G. (1998). 
The use of the stratified-Petersen estimator in fisheries management 
with an illustration of estimating the number of pink salmon 
(Oncorhynchus gorbuscha) that return to spawn in the Fraser River. 
Canadian Journal of Fisheries and Aquatic Sciences, 55, 281–296.
https://doi.org/10.1139/f97-238

---
title: "Harrison River Female Chinook 2011 Data"
author: "Carl James Schwarz"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    number_sections: yes
    toc: yes
    md_extensions: [ 
      "-autolink_bare_uris" 
    ]
vignette: >
  %\VignetteIndexEntry{Harrison River Female Chinook 2011 Data} 
  %\VignetteEngine{knitr::rmarkdown} 
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction
The experiment consisted of daily sampling, but condensed to weeks.
Returning adult salmon were tagged and released at the upstream trap. They were recaptured
during carcass surveys

A stratified-Petersen estimator (Darroch, 1961; Plante et al. 1996) is found for the unpooled data and some pooling of the rows/columns.

# Load the SPAS package
```{r libraries}
library(SPAS)
```
This will load the SPAS fitting functions and any associated packages needed by SPAS.

# Import the data.
The data  should be stored as an $s+1$ by $t+1$ (before pooling) matrix.
The $s \times t$ upper left matrix is the number of animals released in row stratum $i$ and recovered in
column stratum $j$. 
Row $s+1$ contains the total number of UNMARKED animals recovered in column stratum $j$.
Column $t+1$ contains the number of animals marked in each row stratum but not recovered in any column stratum.
The $rawdata[s+1, t+1]$ is not used and can be set to 0 or NA.
The sum of the entries in each of the first $s$ rows is then the number of animals marked in each row stratum.
The sum of the entries in each of the first $t$ columns is then the number of animals captured (marked and unmarked) in each column stratum.
The row/column names of the matrix may be set to identify the entries in the output.

Here is the raw data for the Harrison River. Notice that very small number of releases and recoveries in week 6. 

```{r loaddata}
harrison.2011.chinook.F.csv <- textConnection("
  4   ,      2   ,      1   ,     1   ,     0   ,     0   ,   130
 12   ,      7   ,     14   ,     1   ,     3   ,     0   ,   330
  7   ,     11   ,     41   ,     9   ,     1   ,     1   ,   790
  1   ,     13   ,     40   ,    12   ,     9   ,     1   ,   667
  0   ,      1   ,      8   ,     8   ,     3   ,     0   ,   309
  0   ,      0   ,      0   ,     0   ,     0   ,     1   ,    65
744   ,   1187   ,   2136   ,   951   ,   608   ,   127   ,     0")

har.data <- as.matrix(read.csv(harrison.2011.chinook.F.csv, header=FALSE))
har.data
```
A total of `r sum(har.data[1,])` fish were tagged and released in week 1. Of these fish, `r har.data[1,1]` were recovered down stream in column stratum 1;
`r har.data[1,2]` were recovered in column stratum 2; and `r har.data[1,7]` were never seen again.
A total of `r har.data[7,1]` UNMARKED fish were recovered in column stratum 1.

You can add row and column names to the matrix which will used when the data are printed.

# Fitting the Stratified-Petersen model
The Stratified-Petersen model is fit to the above data object.
The very sparse last row will
make fitting the model to the original data difficult with a very flat likelihood surface and so we relax the convergence
criteria:
```{r fit1,results="hide"}
mod..1 <- SPAS.fit.model(har.data,
                       model.id="No restrictions",
                       row.pool.in=1:6, col.pool.in=1:6)
```
The `row.pool.in` and `col.pool.in` inform the function on which rows or columns to pool before the analysis proceeds. 
Both parameters use a vector of codes (length $s$ for row pooing and length $t$ for column pooling) where rows/columns
are pooled that share the same value.

For example. `row.pool.in=c(1,2,3,4,5,6)` would imply that no rows are pooled, while
`row.pool.in=c('a','a','a','b','b','b')` would imply that the first three rows and last three rows are pooled.
The entries in the vector can be numeric or character; however, using character entries implies that the final
pooled matrix is displayed in the order of the character entries. I find that using entries such as `'123'` to represent
pooling rows 1, 2, and 3 to be easiest to use.

**The SPAS system only fits models were the number of rows after pooling is less than or equal to the number of columns after pooling.**

## Results from model 1 (no pooling).
The results of the model fit is a LARGE list. But the function `SPAS.print.model` produces a nice report
```{r mod1p}
SPAS.print.model(mod..1)
```

The original data, the data after pooling, estimates and their standard errors are shown.
Here the stratified-Petersen estimate of the total number of smolts passing the at the first sampling station is
`r formatC(round(mod..1$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod..1$se $real$N), format='f', digits=0, big.mark=',')`.


Each entry in the output is available in fitted model object. You will need to explore the
structure
```{r str1}
cat("Names of objects at highest level\n")
names(mod..1)
cat("\n\nNames of estimates (both beta and real)\n")
names(mod..1$est)
cat("\n\nNames of real estimates\n")
names(mod..1$est$real)
```
The `N` entries refer to the  population size; the `N.stratum` entries refer to the individual stratum population sizes;
the `cap` entries refer to the estimated probability of capture in each row stratum;
the `exp.factor` entries refer to `(1-cap)/cap`, or the expansion factor for each row;
the `psi` entries refer to the number of animals tagged but never seen again (the right most column in the input data);
and the `theta` entries refer to the expected number of animals that were tagged in row stratum $i$ and recovered in column stratum $j$ (after pooling).

# Pooling some rows and columns
As noted by Darroch (1961), the stratified-Petersen will fail if the matrix of movements is close to singular. This often happens if two rows are proportional to each other. In this case, there is no unique MLE for the probability of capture in the two rows, and they should be pooled.
A detailed discussion of pooling is found in Schwarz and Taylor (1998).

## Physically pooling first two rows and last two rows
Let us now pool the first two rows and the last two rows, but leave rows 3 and 4 alone.
Similar, let us pool the last two columns.

The code is
```{r fit2,results="hide"}
mod..2 <- SPAS.fit.model(har.data, model.id="Pooling some rows",
                       row.pool.in=c("12","12","3","4","56","56"),
                       col.pool.in=c(1,2,3,4,56,56))
```

Notice how we specify the pooling for rows and columns and the choice of entries for the two corresponding vectors.
I used character entries for the row pooling to ensure that the pooled matrix is sorted properly (see below);
the numeric entries for the columns is ok as is.

The results of the model fit is
```{r mod2p}
SPAS.print.model(mod..2)
```

Here the stratified-Petersen estimate of the total number of smolts passing the at the first sampling station is
`r formatC(round(mod..2$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod..2$est$real$N), format='f', digits=0, big.mark=',')`.
which is a slight reduction
from the unpooled estimates.

## Physical pooling to a single row and complete pooling
You can physically pool to a single row (and multiple columns) or a single row and single column (which is equivalent to the pooled Petersen estimator).
The code and output follow:

```{r mod3, echo=TRUE}
mod..3 <- SPAS.fit.model(har.data, model.id="A single row",
                       row.pool.in=rep(1, nrow(har.data)-1),
                       col.pool.in=c(1,2,3,4,56,56))
SPAS.print.model(mod..3)
```

```{r }
mod..4 <- SPAS.fit.model(har.data, model.id="Pooled Peteren",
                       row.pool.in=rep(1, nrow(har.data)-1),
                       col.pool.in=rep(1, ncol(har.data)-1))
SPAS.print.model(mod..4)
```
Both models have an estimated abundance of
`r formatC(round(mod..3$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod..3$se $real$N), format='f', digits=0, big.mark=',')`.
which is a slight reduction
from the unpooled estimates.

## Logically pooling the first two rows and last two rows.
Logical pooling is specified the same way as physical pooling except you set
the additional argument *raw.physical.pool=FALSE*.

The code and output are:
```{r }
mod..5 <- SPAS.fit.model(har.data, model.id="Logical Pooling some rows",
                       row.pool.in=c("12","12","3","4","56","56"),
                       row.physical.pool=FALSE,
                       col.pool.in=c(1,2,3,4,56,56))
SPAS.print.model(mod..5)
```


## Logical pooling to a single row and complete pooling
You can also logically pool to a single row (and multiple columns) or a single row and single column (which is equivalent to the pooled Petersen estimator).

Logical pooling is specified the same way as physical pooling except you set
the additional argument *raw.physical.pool=FALSE*.

The code and output follow:

```{r }
mod..6 <- SPAS.fit.model(har.data, model.id="A single row - Logical Pool",
                       row.pool.in=rep(1, nrow(har.data)-1), row.physical.pool=FALSE,
                       col.pool.in=c(1,2,3,4,56,56))

SPAS.print.model(mod..6)

mod..7 <- SPAS.fit.model(har.data, model.id="Pooled Peteren - Logical Pool",
                       row.pool.in=rep(1, nrow(har.data)-1), row.physical.pool=FALSE,
                       col.pool.in=rep(1, ncol(har.data)-1))

SPAS.print.model(mod..7)
```
Both models have an estimated abundance of
`r formatC(round(mod..5$est$real$N), format='f', digits=0, big.mark=',')` with a standard error of 
`r formatC(round(mod..5$se $real$N), format='f', digits=0, big.mark=',')`.
which is a slight reduction
from the unpooled estimates.  

Note that the estimates from physical and logical pooling are the same.


## Logically pooling every successive pair of rows
Looking at the results from the first 6 models, it appears that rows
3 and 4 are also approximately proportional and should be pooled.

The code and output are:
```{r }
mod..8 <- SPAS.fit.model(har.data, model.id="Logical Pooling pairs rows",
                       row.pool.in=c("12","12","34","34","56","56"),
                       row.physical.pool=FALSE,
                       col.pool.in=c(1,2,3,4,56,56))
SPAS.print.model(mod..8)
```



# Comparing the different estimates:
We extract the results from the models and make a summary report:

```{r echo=TRUE}
model.list <- mget( ls()[grepl("^mod\\.\\.",ls())])
names(model.list)

report <- plyr::ldply(model.list, function(x){
   #browser()
   data.frame(#version=x$version,
              date   = as.Date(x$date),
              model.id         = x$model.info$model.id,
              s.a.pool         =-1+nrow(x$fit.setup$pooldata),
              t.p.pool         =-1+ncol(x$fit.setup$pooldata),
              logL.cond        = x$model.info$logL.cond,
              np               = x$model.info$np,
              AICc             = x$model.info$AICc,
              gof.chisq        = round(x$gof$chisq,1),
              gof.df           = x$gof$chisq.df,
              gof.p            = round(x$gof$chisq.p,3),
              kappa.after.lp   = round(x$kappa.after.lp),
              Nhat             = round(x$est$real$N),
              Nhat.se          = round(x$se $real$N))
  
})
report

```




**Unfortunately, because pooling actually changes the data (internally), it is not possible to
do likelihood ratio testing or to use AICc to compare different physical poolings.**

Different logical poolings can be compared without problems. In this case, models 1, 5, 6 and 8 can be compared because they involve no pooling, or logical pooling of the rows. Model 8 has the (arithmetically) smallest AIC but is not much preferred over
model 6 because the difference in AIC is small.

Both of these model have adequate goodness of fit.
The *kappa.after.lp* for both models is adequate.

In this case, all of methods that lead to a single row give the same estimated population size and standard error and all are equivalent to the Pooled Petersen estimator. 
The Chapman modification to the Petersen estimator is also given in the output and
results are similar.


Different column pooling will give the same fit and so cannot be compared. Refer
to the vignette on why column poolings cannot be compared for more details.


# References
Darroch, J. N. (1961). The two-sample capture-recapture census when tagging and sampling are stratified. Biometrika, 48, 241–260.
https://www.jstor.org/stable/2332748

Plante, N., L.-P Rivest, and G. Tremblay. (1988). Stratified Capture-Recapture Estimation of the Size of a Closed Population. Biometrics 54, 47-60.
https://www.jstor.org/stable/2533994

Schwarz, C. J., & Taylor, C. G. (1998). The use of the stratified-Petersen estimator in fisheries management with an illustration of estimating the number of pink salmon (Oncorhynchus gorbuscha) that return to spawn in the Fraser River. Canadian Journal of Fisheries and Aquatic Sciences, 55, 281–296.
https://doi.org/10.1139/f97-238

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPAS.extract.par.est.R
\name{SPAS.extract.par.est}
\alias{SPAS.extract.par.est}
\title{Extract the estimates into a simpler form from a Stratified-Petersen fit}
\usage{
SPAS.extract.par.est(est.red, rowDM, colDM, thetaDM, conditional = TRUE)
}
\arguments{
\item{est.red}{Vector of parameter estimates}

\item{rowDM}{Design matrix for covariates for row parameters (after pooling). Not currently implemented so
this is usually an identity matrix.}

\item{colDM}{Design matrix for covariates for column parameters (after pooling). Not currently implemented so
this is usually an identity matrix.}

\item{thetaDM}{Design matrix for covariates for movement parameters (after pooling). Not currently implemented so
this is usually an identity matrix.}

\item{conditional}{Indicates if a conditional (Poisson model) or full likelihood (Multinomial) fit was done.}
}
\value{
Returns a list with entries 
\itemize{
  \item theta The modeled number of animals released in row stratum i and recovered in column stratum j.
  \item cap. The estimated probability of tagging in row stratum i.
  \item psi. The estimated number of animals never seen after tagging for each row stratum.
  \item N.   The estimated population size.
}
}
\description{
This function is normally used internally and not by users of the package.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPAS.misc.functions.r
\name{logit}
\alias{logit}
\title{Helper functions for this package (logit, expit)}
\usage{
logit(p)
}
\description{
These functions are helper functions for the SPAS package and not normally accessed by the user
}
\details{
\code{logit} - compute the logit of a proportion
\code{expit} - compute a proportion from a logit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPAS.autopool.R
\name{SPAS.autopool}
\alias{SPAS.autopool}
\title{Autopooling a Stratified-Petersen (SP) data set.
This function applies pooling rules to pool a SPAS dataset to meeting minimum sparsity requirements
.}
\usage{
SPAS.autopool(
  rawdata,
  min.released = 100,
  min.inspected = 50,
  min.recaps = 50,
  min.rows = 1,
  min.cols = 1
)
}
\arguments{
\item{rawdata}{An (s+1) x (t+1) of the raw data BEFORE pooling.
The s x t upper left matrix is the number of animals released in row stratum i and recovered in
column stratum j. Row s+1 contains the total number of UNMARKED animals recovered in column stratum j.
Column t+1 contains the number of animals marked in each row stratum but not recovered in any column stratum.
The rawdata[s+1, t+1] is not used and can be set to 0 or NA.
The sum of the entries in each of the first s rows is then the number of animals marked in each row stratum.
The sum of the entries in each of the first t columns is then the number of animals captured (marked and unmarked) in each column stratum.
The row/column names of the matrix may be set to identify the entries in the output.}

\item{min.released}{Minimum number of releases in a pooled row}

\item{min.inspected}{Minimum number of inspections in a pooled column}

\item{min.recaps}{Minimum number of recaptures before any rows can be pooled}

\item{min.rows, min.cols}{Minimum number or rows and columns after pooling}
}
\value{
A list with a suggest pooling.
}
\description{
Autopooling a Stratified-Petersen (SP) data set.
This function applies pooling rules to pool a SPAS dataset to meeting minimum sparsity requirements
.
}
\details{
In many cases, the stratified set of releases and recapture is too sparse (many zeroes) or count
are very small. Pooling rows and columns may be needed.

Data needs to be pooled both row wise and column wise if the data are sparse
to avoid singularities in the fit.
This function automates pooling rows or columns following Schwarz and Taylor (1998).
\itemize{
\item All rows that have 0 releases are discarded
\item All columns that have 0 recaptures of tagged fish and 0 fish inspected are discarded
\item Starting at the first row and working forwards in time,
and then working from the final row and working backwards in time,
.  rows are pooled until a minimum of \code{min.released} are released.
An alternating pooling (from the top, from the bottom, from the top, etc) is used
\item Starting at the first column and working forwards in time,
.   and then working from the final column and working backwards in time,
columns are pooled until a minimum of \code{min.inspected} are inspected.
An alternating pooling (from the left, from the right, from the left, etc) is used.
\item If the sum of the total recaptures from released fish is <= \code{min.recaps}, then all rows are pooled
(which reduces to a Chapman estimator)
}
}
\examples{
conne.data.csv <- textConnection("
9  ,    21  ,     0  ,    0  ,    0  ,    0  ,   171
0  ,   101  ,    22  ,    1  ,    0  ,    0  ,   763
0  ,     0  ,   128  ,   49  ,    0  ,    0  ,   934
0  ,     0  ,     0  ,   48  ,   12  ,    0  ,   434
0  ,     0  ,     0  ,    0  ,    7  ,    0  ,    49
0  ,     0  ,     0  ,    0  ,    0  ,    0  ,     4
351,  2736  ,  3847  , 1818  ,  543  ,   191 ,     0")
conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
close(conne.data.csv)
 
SPAS.autopool(conne.data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/roxygentags.R
\name{dummy}
\alias{dummy}
\title{Roxygen commands}
\usage{
dummy()
}
\description{
Roxygen commands
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{.onAttach}
\alias{.onAttach}
\title{Message to display when package is loaded}
\usage{
.onAttach(libname, pkgname)
}
\description{
Message to display when package is loaded
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPAS.print.model.R
\name{SPAS.print.model}
\alias{SPAS.print.model}
\title{Print or Extract the results from a fit of a Stratified-Petersen (SP) model when using the TMB optimizer}
\usage{
SPAS.print.model(x, extract = FALSE)
}
\arguments{
\item{x}{A result from the model fitting. See \code{SPAS.fit.model}}

\item{extract}{Should the function simply return the "printed" components in a list?.}
}
\value{
A report to the console or extracted components. Refer to the vignettes.
}
\description{
This function makes a report/extracts the components of the results of the model fitting
.
}
\examples{
conne.data.csv <- textConnection("
9  ,    21  ,     0  ,    0  ,    0  ,    0  ,   171
0  ,   101  ,    22  ,    1  ,    0  ,    0  ,   763
0  ,     0  ,   128  ,   49  ,    0  ,    0  ,   934
0  ,     0  ,     0  ,   48  ,   12  ,    0  ,   434
0  ,     0  ,     0  ,    0  ,    7  ,    0  ,    49
0  ,     0  ,     0  ,    0  ,    0  ,    0  ,     4
351,  2736  ,  3847  , 1818  ,  543  ,   191 ,     0")
conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
close(conne.data.csv)
 
mod1 <- SPAS.fit.model(conne.data, model.id="Pooling rows 1/2, 5/6; pooling columns 5/6",
                      row.pool.in=c("12","12","3","4","56","56"),
                      col.pool.in=c(1,2,3,4,56,56))

SPAS.print.model(mod1)

out <- SPAS.print.model(mod1, extract=TRUE)
names(out)
out$spas
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPAS.fit.model.R
\name{SPAS.fit.model}
\alias{SPAS.fit.model}
\title{Fit a Stratified-Petersen (SP) model using  TMB.}
\usage{
SPAS.fit.model(
  model.id = "Stratified Petersen Estimator",
  rawdata,
  autopool = FALSE,
  row.pool.in = NULL,
  col.pool.in = NULL,
  row.physical.pool = TRUE,
  theta.pool = FALSE,
  CJSpool = FALSE,
  optMethod = c("nlminb"),
  optMethod.control = list(maxit = 50000),
  svd.cutoff = 1e-04,
  chisq.cutoff = 0.1,
  min.released = 100,
  min.inspected = 50,
  min.recaps = 50,
  min.rows = 1,
  min.cols = 1
)
}
\arguments{
\item{model.id}{Character string identifying the name of the model including any pooling..}

\item{rawdata}{An (s+1) x (t+1) of the raw data BEFORE pooling.
The s x t upper left matrix is the number of animals released in row stratum i and recovered in
column stratum j. Row s+1 contains the total number of UNMARKED animals recovered in column stratum j.
Column t+1 contains the number of animals marked in each row stratum but not recovered in any column stratum.
The rawdata[s+1, t+1] is not used and can be set to 0 or NA.
The sum of the entries in each of the first s rows is then the number of animals marked in each row stratum.
The sum of the entries in each of the first t columns is then the number of animals captured (marked and unmarked) in each column stratum.
The row/column names of the matrix may be set to identify the entries in the output.}

\item{autopool}{Should the automatic pooling algorithms be used.
Give more details here on these rule work.}

\item{row.pool.in, col.pool.in}{Vectors (character/numeric) of length s and t respectively. These identify the rows/columns to be pooled before the analysis is done.
The vectors consists of entries where pooling takes place if the entries are the same. For example, if s=4, then 
row.pool.in = c(1,2,3,4) implies no pooling because all entries are distinct; row.pool.in=c("a","a","b","b") implies that the 
first two rows will be pooled and the last two rows will be pooled. It is not necessary that row/columns be continuous to be pooled, but
this is seldom sensible. A careful choice of pooling labels helps to remember what as done, e.g. row.pool.in=c("123","123","123","4") indicates
that the first 3 rows are pooled and the 4th row is not pooled. Character entries ensure that the resulting matrix is sorted properly (e.g. if 
row.pool.in=c(123,123,123,4), then the same pooling is done, but the matrix rows are sorted rather strangely.}

\item{row.physical.pool}{Should physical pooling be done (default) or should logical pooling be done. For example, if there are 3 rows in 
the data matrix and row.pool.in=c(1,1,3), then in physical pooling, the entries in rows 1 and 2 are physically added together to create
2 rows in the data matrix before fitting. Because the data has changed, you cannot compare physical pooling using AIC. In logical pooling,
the data matrix is unchanged, but now parameters p1=p2 but the movement parameters for the rest of the matrix are not forced equal.}

\item{theta.pool, CJSpool}{NOT YET IMPLEMENTED. DO NOT CHANGE.}

\item{optMethod}{What optimization method is used. Defaults is the nlminb() function..}

\item{optMethod.control}{Control parameters for optimization method. See the documentation on the different optimization methods for details.}

\item{svd.cutoff}{When finding the variance-covariance matrix, a singular value decomposition is used. This identifies the smallest 
singular value to retain.}

\item{chisq.cutoff}{When finding a goodness of fit statistic using (obs-exp)^2/exp, all cell whose Exp < gof.cutoff are ignored
to try and remove structural zero cells.}

\item{min.released}{Minimum number of releases in a pooled row}

\item{min.inspected}{Minimum number of inspections in a pooled column}

\item{min.recaps}{Minimum number of recaptures before any rows can be pooled}

\item{min.rows, min.cols}{Minimum number or rows and columns after pooling}
}
\value{
A list with many entries. Refer to the vignettes for more details.
}
\description{
This function fits a Stratified-Petersen (Plante, 1996) to data and specify which rows/columns of the data
should be pooled. The number of rows after pooling should be <= number of columns after
pooling
.
}
\examples{
conne.data.csv <- textConnection("
9  ,    21  ,     0  ,    0  ,    0  ,    0  ,   171
0  ,   101  ,    22  ,    1  ,    0  ,    0  ,   763
0  ,     0  ,   128  ,   49  ,    0  ,    0  ,   934
0  ,     0  ,     0  ,   48  ,   12  ,    0  ,   434
0  ,     0  ,     0  ,    0  ,    7  ,    0  ,    49
0  ,     0  ,     0  ,    0  ,    0  ,    0  ,     4
351,  2736  ,  3847  , 1818  ,  543  ,   191 ,     0")
conne.data <- as.matrix(read.csv(conne.data.csv, header=FALSE))
close(conne.data.csv)
 
mod1 <- SPAS.fit.model(conne.data, model.id="Pooling rows 1/2, 5/6; pooling columns 5/6",
                      row.pool.in=c("12","12","3","4","56","56"),
                      col.pool.in=c(1,2,3,4,56,56))
mod2 <- SPAS.fit.model(conne.data, model.id="Auto pool",
                      autopool=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SPAS.log.likelihood.R
\name{SPAS.likelihood.star.DM}
\alias{SPAS.likelihood.star.DM}
\title{Score, likelihood, and related functions for fitting Stratified Petersen model}
\usage{
SPAS.likelihood.star.DM(
  Est.Star,
  rowDM,
  colDM,
  thetaDM,
  rawdata,
  returnnegll = FALSE,
  conditional = TRUE,
  fixed.N = 0,
  mytrace = FALSE
)
}
\description{
These functions are used internally to fit the Stratified Petersen models and are not normally used.
Most of these functions are for the legacy version that does not use TMB and so likely can be eventually deleted.
}
\keyword{internal}
